@@ -68,8 +68,6 @@ class AuthenticationBlock:
                 readable string.
         """
         self.log = BlockLogger(log_output=log_output)
-        self.block_label = "Authenticating"
-        self.log.open_block(self.block_label)
         self.host = host or shared_anyscale_conf.ANYSCALE_HOST
         if cli_token is not None:
             self.credentials = cli_token
@@ -93,7 +91,6 @@ class AuthenticationBlock:
             f"Loaded Anyscale authentication token from {cli_token_location}.",
         )
         self._warn_credential_file_permissions(cli_token_location)
-        self.log.close_block(self.block_label)
 
     @property
     def product_api_client(self) -> DefaultApi:
@@ -114,6 +111,7 @@ class AuthenticationBlock:
                 readable string.
         """
         configuration = openapi_client.Configuration(host=self.host)
+        configuration.proxy = os.environ.get("https_proxy")
         configuration.connection_pool_maxsize = 100
 
         cookie = f"cli_token={self.credentials}"
@@ -141,6 +139,7 @@ class AuthenticationBlock:
                 a threadpool. Invocations of the api client will return coroutines.
         """
         configuration = anyscale_client.Configuration(host=self.host + "/ext/v0")
+        configuration.proxy = os.environ.get("https_proxy")
         configuration.connection_pool_maxsize = 100
 
         api_client = (
