@@ -1,7 +1,7 @@
 import json
 from typing import Any, Dict
 
-from pydantic import BaseModel
+from anyscale.anyscale_pydantic import BaseModel
 
 
 # Used for data-gplane role.
