@@ -1,4 +1,5 @@
 import difflib
+import json
 import pprint
 from typing import Any, Dict, List, Optional
 
@@ -24,7 +25,7 @@ from anyscale.client.openapi_client.models.subnet_id_with_availability_zone_aws
     SubnetIdWithAvailabilityZoneAWS,
 )
 from anyscale.shared_anyscale_utils.aws import AwsRoleArn
-from anyscale.shared_anyscale_utils.conf import ANYSCALE_HOST
+from anyscale.shared_anyscale_utils.conf import ANYSCALE_CORS_ORIGIN
 from anyscale.util import (  # pylint:disable=private-import
     _get_subnet,
     contains_control_plane_role,
@@ -45,6 +46,8 @@ from anyscale.utils.s3 import verify_s3_access
 # It should go away with the SSM refactor.
 DEFAULT_RAY_IAM_ROLE = "ray-autoscaler-v1"
 
+S3_ARN_PREFIX = "arn:aws:s3:::"
+
 
 def compare_dicts_diff(d1: Dict[Any, Any], d2: Dict[Any, Any]) -> str:
     """Returns a string representation of the difference of the two dictionaries.
@@ -512,7 +515,7 @@ def verify_aws_security_groups(  # noqa: PLR0912, PLR0911
             missing_open_ports.append(port)
     if missing_open_ports:
         logger.warning(
-            f"Security groups {aws_security_group_ids} do not contain inbound permission for ports: {missing_open_ports}. These ports are used for interaction with the clusters from Anyscale UI. Please make sure to configure them according to https://docs.anyscale.com/user-guide/onboard/clouds/deploy-on-aws#appendix-detailed-resource-requirements"
+            f"Security groups {aws_security_group_ids} do not contain inbound permission for ports: {missing_open_ports}. These ports are used for interaction with the clusters from Anyscale UI. Please make sure to configure them according to https://docs.anyscale.com/cloud-deployment/aws/manage-clouds#appendix-detailed-resource-requirements"
         )
         if strict:
             return False
@@ -618,7 +621,7 @@ def verify_aws_s3(  # noqa: PLR0911, PLR0912
             logger.error(f"Malformed CORS rule {rule} for your S3 bucket.")
             return False
         has_correct_cors_rule = (
-            ANYSCALE_HOST in rule.get("AllowedOrigins", [])
+            ANYSCALE_CORS_ORIGIN in rule.get("AllowedOrigins", [])
             and "*" in rule.get("AllowedHeaders", [])
             and "GET" in rule.get("AllowedMethods", [])
         )
@@ -629,7 +632,7 @@ def verify_aws_s3(  # noqa: PLR0911, PLR0912
             CloudSetupError.INCORRECT_CORS_RULE,
         )
         logger.warning(
-            f"S3 bucket {bucket_name} does not have the correct CORS rule for Anyscale. This is safe to ignore if you are not using Anyscale UI. Otherwise please create the correct CORS rule for Anyscale according to https://docs.anyscale.com/anyscale-cloud-administration/deploy-on-aws#appendix-detailed-resource-requirements"
+            f"S3 bucket {bucket_name} does not have the correct CORS rule for Anyscale. This is safe to ignore if you are not using Anyscale UI. Otherwise please create the correct CORS rule for Anyscale according to https://docs.anyscale.com/cloud-deployment/aws/manage-clouds#appendix-detailed-resource-requirements"
         )
         if strict:
             return False
@@ -836,10 +839,56 @@ def verify_aws_efs(  # noqa: PLR0911, PLR0912, C901
         else:
             raise e
 
+    # Verify efs policy
+    if not _verify_aws_efs_policy(
+        boto3_session, cloud_resource.aws_efs_id, logger, strict
+    ):
+        return False
+
     logger.info(f"EFS {cloud_resource.aws_efs_id} verification succeeded.")
     return True
 
 
+def _verify_aws_efs_policy(
+    boto3_session: boto3.Session,
+    efs_id: str,
+    logger: CloudSetupLogger,
+    strict: bool = False,
+) -> bool:
+    """
+    Verify that the EFS policy has sufficient permissions.
+    """
+    client = boto3_session.client("efs")
+
+    efs_policy = None
+    try:
+        efs_policy_response = client.describe_file_system_policy(FileSystemId=efs_id)
+        efs_policy = json.loads(efs_policy_response["Policy"])
+    except ClientError as e:
+        if e.response["Error"]["Code"] != "PolicyNotFound":
+            # If the policy is not found, we'll skip the check
+            logger.error(f"Failed to describe file system policy: {e}")
+            return False
+
+    if efs_policy:
+        expected_actions = [
+            "elasticfilesystem:ClientRootAccess",
+            "elasticfilesystem:ClientWrite",
+            "elasticfilesystem:ClientMount",
+        ]
+        actions = filter_actions_from_policy_document(efs_policy)
+        missing_actions = set(expected_actions) - set(actions)
+        if missing_actions:
+            logger.warning(
+                f"EFS {efs_id} does not have sufficient permissions. "
+                f"We suggest adding these actions to ensure that efs works properly: {missing_actions}. "
+            )
+            if strict:
+                return False
+
+    return True
+
+
 def verify_aws_cloudformation_stack(
     cloud_resource: CreateCloudResource,
     boto3_session: boto3.Session,
