@@ -1,4 +1,5 @@
-from typing import Optional
+import re
+from typing import Optional, Tuple
 
 from anyscale.authenticate import get_auth_api_client
 from anyscale.cli_logger import BlockLogger
@@ -46,6 +47,36 @@ def get_default_cluster_compute(
     return compute_template
 
 
+def parse_cluster_compute_name_version(
+    cluster_compute_name_version: str,
+) -> Tuple[str, Optional[int]]:
+
+    # Regex pattern to validate the cluster compute name version string:
+    # - the name could not contain colon, leading or trailing spaces
+    # - the version should be a positive integer
+    # - the version is optional
+    # - the name and version should be separated by a colon if version is provided
+    COMPUTE_CONFIG_NAME_VERSION_VALIDATION_REGEX_PATTERN = (
+        r"^[^\s:]+(\s+[^\s:]+)*(:[1-9]+[0-9]*)?$"
+    )
+
+    if not re.match(
+        COMPUTE_CONFIG_NAME_VERSION_VALIDATION_REGEX_PATTERN,
+        cluster_compute_name_version,
+    ):
+        raise ValueError(
+            f"Invalid compute config name and version '{cluster_compute_name_version}'. Please provide name and version in format `my-compute-config:3` or provide compute configuration name `my-compute-config`."
+        )
+
+    cluster_compute_name = cluster_compute_name_version
+    version = None
+    if cluster_compute_name_version.count(":") == 1:
+        cluster_compute_name, parsed_version = cluster_compute_name_version.split(":")
+        version = int(parsed_version)
+
+    return cluster_compute_name, version
+
+
 def get_cluster_compute_from_name(
     cluster_compute_name: str,
     api_client: Optional[DefaultApi] = None,
@@ -53,6 +84,30 @@ def get_cluster_compute_from_name(
 ) -> ComputeTemplate:
     if api_client is None:
         api_client = get_auth_api_client().api_client
+
+    enable_compute_config_versioning = api_client.check_is_feature_flag_on_api_v2_userinfo_check_is_feature_flag_on_get(
+        "compute-config-versioning"
+    ).result.is_on
+
+    if enable_compute_config_versioning:
+        (
+            parsed_cluster_compute_name,
+            parsed_version,
+        ) = parse_cluster_compute_name_version(cluster_compute_name)
+
+        if parsed_version and version:
+            # TODO: after cleaning up the separate version argument, remove this check.
+            raise ValueError(
+                "Please provide either version in the name or as a separate argument, not both."
+            )
+        elif parsed_version:
+            cluster_compute_name = parsed_cluster_compute_name
+            version = parsed_version
+        else:
+            # version is not None and parsed_version is None.
+            # Use non-parsed cluster_compute_name and version.
+            pass
+
     cluster_computes = api_client.search_compute_templates_api_v2_compute_templates_search_post(
         ComputeTemplateQuery(
             orgwide=True,
@@ -65,7 +120,7 @@ def get_cluster_compute_from_name(
 
     if len(cluster_computes) == 0:
         raise ValueError(
-            f"The cluster compute template {cluster_compute_name} is not registered."
+            f"The compute config {cluster_compute_name} does not exist or you don't have sufficient permissions."
         )
     return cluster_computes[0]
 
