@@ -60,16 +60,18 @@ def cluster_cli() -> None:
     "--compute",
     required=False,
     default=None,
-    help=("Name of cluster compute that is already registered with Anyscale."),
+    help=(
+        "Name of compute config that is already registered with Anyscale. To use specific version, use the format `compute_name:version`."
+    ),
 )
 @click.option(
     "--compute-file",
     required=False,
     default=None,
     help=(
-        "The YAML file of the cluster compute config to launch this cluster with. "
+        "The YAML file of the compute config to launch this cluster with. "
         "An example can be found at {website}. ".format(
-            website="https://docs.anyscale.com/user-guide/configure/cluster-computes",
+            website="https://docs.anyscale.com/configure/compute-configs/overview",
         )
     ),
 )
@@ -106,7 +108,7 @@ def cluster_cli() -> None:
     required=False,
     default=None,
     help=(
-        "Name of cloud to create a default cluster compute with. If a default "
+        "Name of cloud to create a default compute config with. If a default "
         "cloud needs to be used and this is not provided, the organization default "
         "cloud will be used."
     ),
@@ -115,7 +117,7 @@ def cluster_cli() -> None:
     "--idle-timeout",
     required=False,
     default=None,
-    help="DEPRECATED: Please specify the idle_termination_minutes field in the cluster compute. "
+    help="DEPRECATED: Please specify the idle_termination_minutes field in the compute config. "
     "Idle timeout (in minutes), after which the cluster is stopped. Idle "
     "time is defined as the time during which a cluster is not running a user "
     "command and does not have an attached driver. Time spent running Jupyter "
@@ -136,17 +138,6 @@ def cluster_cli() -> None:
         "the same Anyscale cloud and will not require an authentication token."
     ),
 )
-@click.option(
-    "--cluster-compute-version",
-    required=False,
-    default=None,
-    type=int,
-    help=(
-        "The cluster compute version that you want to use. Only applicable if the compute was specified by name. "
-        "Versions are positive integers. "
-        "Deprecated behavior: Not setting a version is equivalent to setting version to '1'."
-    ),
-)
 def start(  # noqa: PLR0913
     name: Optional[str],
     env: Optional[str],
@@ -161,7 +152,6 @@ def start(  # noqa: PLR0913
     cloud_name: Optional[str],
     idle_timeout: Optional[int],
     user_service_access: Optional[str],
-    cluster_compute_version: Optional[int],
 ) -> None:
     cluster_controller = ClusterController()
     if HIDE_BYOD_FLAGS:
@@ -187,7 +177,6 @@ def start(  # noqa: PLR0913
         project_id=project_id,
         project_name=project,
         user_service_access=user_service_access,
-        cluster_compute_version=cluster_compute_version,
     )
 
 
