@@ -101,3 +101,25 @@ def get(cluster_env_name: Optional[str], cluster_env_build_id: Optional[str]) ->
     cluster_env_controller.get(
         cluster_env_name=cluster_env_name, build_id=cluster_env_build_id,
     )
+
+
+@cluster_env_cli.command(
+    name="archive", help=("Archive the specified cluster environment."),
+)
+@click.option(
+    "--name",
+    "-n",
+    help="Name of the cluster environment to archive.",
+    required=False,
+    type=str,
+)
+@click.option(
+    "--cluster-env-id",
+    "--id",
+    help="Id of the cluster environment to archive. Must be provided if a cluster environment name is not given.",
+    required=False,
+    type=str,
+)
+def archive(name: Optional[str], cluster_env_id: Optional[str]) -> None:
+    cluster_env_controller = ClusterEnvController()
+    cluster_env_controller.archive(name, cluster_env_id)
