@@ -23,7 +23,7 @@ def compute_config_cli() -> None:
         "at https://docs.anyscale.com/reference/python-sdk/models#createclustercomputeconfig"
     ),
 )
-@click.argument("cluster-compute-file", type=click.File("rb"), required=True)
+@click.argument("compute-config-file", type=click.File("rb"), required=True)
 @click.option(
     "--name",
     "-n",
@@ -31,16 +31,14 @@ def compute_config_cli() -> None:
     required=False,
     type=str,
 )
-def create_cluster_compute(
-    cluster_compute_file: IO[bytes], name: Optional[str]
-) -> None:
-    ComputeConfigController().create(cluster_compute_file, name)
+def create_compute_config(compute_config_file: IO[bytes], name: Optional[str]) -> None:
+    ComputeConfigController().create(compute_config_file, name)
 
 
 @compute_config_cli.command(
     name="delete", help="Delete the specified compute config.", hidden=True
 )
-@click.argument("cluster-compute-name", type=str, required=False)
+@click.argument("compute-config-name", type=str, required=False)
 @click.option(
     "--name",
     "-n",
@@ -50,27 +48,40 @@ def create_cluster_compute(
 )
 @click.option(
     "--cluster-compute-id",
+    help="Deprecated. Use --compute-config-id instead. Id of the compute config to archive. Must be provided if a compute name is not given.",
+    required=False,
+    type=str,
+)
+@click.option(
+    "--compute-config-id",
     "--id",
     help="Id of the compute config to delete. Must be provided if a compute name is not given.",
     required=False,
     type=str,
 )
-def delete_cluster_compute(
-    cluster_compute_name: Optional[str],
+def delete_compute_config(
+    compute_config_name: Optional[str],
     name: Optional[str],
     cluster_compute_id: Optional[str],
+    compute_config_id: Optional[str],
 ) -> None:
-    if cluster_compute_name is not None and name is not None:
+    if compute_config_name is not None and name is not None:
         raise click.ClickException(
-            "Please only provide one of [CLUSTER_COMPUTE_NAME] or --name."
+            "Please only provide one of [COMPUTE_CONFIG_NAME] or --name."
         )
-    ComputeConfigController().delete(cluster_compute_name or name, cluster_compute_id)
+
+    if compute_config_id and cluster_compute_id:
+        raise click.ClickException(
+            "Please only provide one of --compute-config-id or --cluster-compute-id."
+        )
+
+    ComputeConfigController().delete(compute_config_name or name, cluster_compute_id)
 
 
 @compute_config_cli.command(
     name="archive", help="Archive the specified compute config.",
 )
-@click.argument("cluster-compute-name", type=str, required=False)
+@click.argument("compute-config-name", type=str, required=False)
 @click.option(
     "--name",
     "-n",
@@ -80,21 +91,36 @@ def delete_cluster_compute(
 )
 @click.option(
     "--cluster-compute-id",
+    help="Deprecated. Use --compute-config-id instead. Id of the compute config to archive. Must be provided if a compute name is not given.",
+    required=False,
+    type=str,
+)
+@click.option(
+    "--compute-config-id",
     "--id",
     help="Id of the compute config to archive. Must be provided if a compute name is not given.",
     required=False,
     type=str,
 )
-def archive_cluster_compute(
-    cluster_compute_name: Optional[str],
+def archive_compute_config(
+    compute_config_name: Optional[str],
     name: Optional[str],
     cluster_compute_id: Optional[str],
+    compute_config_id: Optional[str],
 ) -> None:
-    if cluster_compute_name is not None and name is not None:
+    if compute_config_name is not None and name is not None:
         raise click.ClickException(
-            "Please only provide one of [CLUSTER_COMPUTE_NAME] or --name."
+            "Please only provide one of [COMPUTE_CONFIG_NAME] or --name."
         )
-    entity = format_inputs_to_entity(cluster_compute_name or name, cluster_compute_id)
+
+    if compute_config_id and cluster_compute_id:
+        raise click.ClickException(
+            "Please only provide one of --compute-config-id or --cluster-compute-id."
+        )
+
+    entity = format_inputs_to_entity(
+        compute_config_name or name, compute_config_id or cluster_compute_id
+    )
     ComputeConfigController().archive(entity)
 
 
@@ -114,6 +140,14 @@ def archive_cluster_compute(
 )
 @click.option(
     "--cluster-compute-id",
+    required=False,
+    default=None,
+    help=(
+        "Deprecated. Use --compute-config-id instead. List information about the compute config with this id."
+    ),
+)
+@click.option(
+    "--compute-config-id",
     "--id",
     required=False,
     default=None,
@@ -136,13 +170,20 @@ def archive_cluster_compute(
 def list(  # noqa: A001
     name: Optional[str],
     cluster_compute_id: Optional[str],
+    compute_config_id: Optional[str],
     include_shared: bool,
     max_items: int,
 ) -> None:
     cluster_compute_controller = ComputeConfigController()
+
+    if compute_config_id and cluster_compute_id:
+        raise click.ClickException(
+            "Please only provide one of --compute-config-id or --cluster-compute-id."
+        )
+
     cluster_compute_controller.list(
         cluster_compute_name=name,
-        cluster_compute_id=cluster_compute_id,
+        cluster_compute_id=compute_config_id or cluster_compute_id,
         include_shared=include_shared,
         max_items=max_items,
     )
@@ -151,23 +192,35 @@ def list(  # noqa: A001
 @compute_config_cli.command(
     name="get", help=("Get details about compute configuration."),
 )
-@click.argument("cluster-compute-name", required=False)
-@click.argument("cluster-compute-version", required=False, default=None, type=int)
+@click.argument("compute-config-name", required=False)
 @click.option(
     "--cluster-compute-id",
+    required=False,
+    default=None,
+    help=(
+        "Deprecated. Use --compute-config-id instead. Get details about compute configuration by this id."
+    ),
+)
+@click.option(
+    "--compute-config-id",
     "--id",
     required=False,
     default=None,
     help=("Get details about compute configuration by this id."),
 )
 def get(
-    cluster_compute_name: Optional[str],
+    compute_config_name: Optional[str],
     cluster_compute_id: Optional[str],
-    cluster_compute_version: Optional[int],
+    compute_config_id: Optional[str],
 ) -> None:
     cluster_compute_controller = ComputeConfigController()
+
+    if compute_config_id and cluster_compute_id:
+        raise click.ClickException(
+            "Please only provide one of --compute-config-id or --cluster-compute-id."
+        )
+
     cluster_compute_controller.get(
-        cluster_compute_name=cluster_compute_name,
-        cluster_compute_id=cluster_compute_id,
-        cluster_compute_version=cluster_compute_version,
+        cluster_compute_name=compute_config_name,
+        cluster_compute_id=compute_config_id or cluster_compute_id,
     )
