@@ -7,7 +7,7 @@ from anyscale.models.fine_tune_model import FineTuneConfig
 
 
 @click.group(
-    "fine-tune", help="Interact with fine-tuning jobs running on Anyscale.", hidden=True
+    "fine-tuning", help="Interact with fine-tuning jobs running on Anyscale.",
 )
 def fine_tune_cli() -> None:
     pass
@@ -29,12 +29,33 @@ def fine_tune_cli() -> None:
     default=None,
     help="The suffix of the fine-tuned model.",
 )
+@click.option(
+    "--version",
+    required=False,
+    default=None,
+    help=(
+        "The version of ray-llm expected to be used for the fine-tuned model. "
+        "If not specified, the latest version will be used."
+    ),
+)
+@click.option(
+    "--instance-type",
+    required=False,
+    default=None,
+    help=(
+        "The instance type to use for the fine-tuning job. "
+        "Please refer to docs at https://docs.anyscale.com/endpoints/fine-tuning/get-started for available instance types."
+        "If not specified, the default instance type will be used."
+    ),
+)
 def submit(
     base_model: str,
     train_file: str,
     valid_file: Optional[str],
     cloud_id: str,
     suffix: Optional[str],
+    version: Optional[str],
+    instance_type: Optional[str],
 ) -> None:
     """
     Submits a fine-tuning job.
@@ -50,5 +71,7 @@ def submit(
         valid_file=valid_file,
         cloud_id=cloud_id,
         suffix=suffix,
+        version=version,
+        instance_type=instance_type,
     )
     fine_tune_controller.fine_tune(config=config)
