@@ -1,13 +1,14 @@
-import json
-import sys
-from typing import List, Optional
+import pathlib
+from subprocess import list2cmdline
+from typing import Optional, Tuple
 
 import click
 
+import anyscale
 from anyscale.cli_logger import BlockLogger
 from anyscale.client.openapi_client import HaJobStates
 from anyscale.controllers.job_controller import JobController
-import anyscale.job
+from anyscale.job.models import JobConfig
 from anyscale.util import validate_non_negative_arg
 
 
@@ -19,10 +20,13 @@ def job_cli() -> None:
     pass
 
 
-@job_cli.command(name="submit",)
-@click.argument("job-config-file", required=True)
+@job_cli.command(
+    name="submit", short_help="Submit a job.",
+)
 @click.option("--name", "-n", required=False, default=None, help="Name of the job.")
-@click.option("--description", required=False, default=None, help="Description of job.")
+@click.option(
+    "--description", required=False, default=None, help="Description of the job."
+)
 @click.option(
     "--follow",
     "-f",
@@ -41,55 +45,62 @@ def job_cli() -> None:
     is_flag=True,
     help="Whether to block this CLI command until the job succeeds (or reaches another terminal state)",
 )
-@click.argument(
-    "entrypoint", required=False, nargs=-1,
-)
+@click.argument("entrypoint", required=False, nargs=-1, type=click.UNPROCESSED)
 def submit(
-    job_config_file: str,
-    entrypoint: List[str],
+    entrypoint: Tuple[str],
     name: Optional[str],
     description: Optional[str],
     follow: Optional[bool],
     wait: Optional[bool],
-) -> None:
-    """ Submit a Production Job
+):
+    """Submit a job.
 
-    This function accepts 1 argument, a path to a YAML config file that defines this job.
+    The job config can be specified from a file or inlined in this command:
 
+    $ anyscale job submit config.yaml # Submit from a config file.
 
-    To wait for the Job to complete, use the --wait flag as follows.
-
-    $ anyscale job submit job.yaml --wait
-
-    If running from an Anyscale Workspace, this command also support an alternative syntax to specify the entrypoint inline:
-
-    $ anyscale job submit -- python script.py
-
-    The Cluster Environment and Cluster Config are inferred from the Workspace Configuration if running from a workspace.
+    $ anyscale job submit -- python main.py # Submit from an inlined entrypoint.
 
+    By default, this command submits the job asynchronously and exits. To wait for the job to complete, use the `--wait` flag.
     """
 
+    if len(entrypoint) == 0:
+        raise click.ClickException(
+            "Either a config file or an inlined entrypoint must be provided."
+        )
+
+    config_path: Optional[str] = None
+    if len(entrypoint) == 1:
+        maybe_config_path = entrypoint[0]
+        if pathlib.Path(maybe_config_path).is_file():
+            config_path = entrypoint[0]
+            log.info(f"Submitting job from config file: '{config_path}'.")
+        elif maybe_config_path.endswith(".yaml"):
+            raise click.ClickException(
+                f"Job config file '{maybe_config_path}' not found."
+            )
+
     job_controller = JobController()
-    id = job_controller.submit(  # noqa: A001
-        job_config_file,
-        name,
-        description,
-        entrypoint=entrypoint,
-        is_entrypoint_cmd="--" in sys.argv,
-    )
+    if config_path is not None:
+        job_id = job_controller.submit(config_path, name=name, description=description,)
+    else:
+        job_id = anyscale.job.submit(
+            JobConfig(name=name, entrypoint=list2cmdline(entrypoint))
+        )
+
     if follow or wait:
         job_controller.log.info(
-            "Job is running on Anyscale. Use Ctrl + C "
-            "to interrupt this command at any time without cancelling the job."
+            "Waiting for the job to run. Interrupting this command will not cancel the job."
         )
     else:
         job_controller.log.info(
-            "Use --follow to stream the output of the job when submitting a job."
+            "Use `--wait` to wait for the job to run and stream logs."
         )
+
     if follow:
-        job_controller.logs(id, should_follow=follow)
+        job_controller.logs(job_id, should_follow=follow)
     if wait:
-        job_controller.wait(job_id=id)
+        job_controller.wait(job_id=job_id)
 
 
 @job_cli.command(name="list", help="Display information about existing jobs.")
@@ -238,62 +249,3 @@ def wait(
     job_id = job_controller.wait(
         job_id=job_id, job_name=name, target_state=state, timeout_secs=timeout,
     )
-
-
-@job_cli.group(
-    "output",
-    help="DEPRECATED: Interact with Production Job outputs. Job outputs do not work in V2 stack and has been removed from the UI. The CLI commands will be removed in the future.",
-)
-def output_cli() -> None:
-    pass
-
-
-@output_cli.command(name="get")
-@click.option("--job-id", "--id", required=False, help="Unique ID of the job.")
-@click.option("--name", "-n", required=False, help="Name of the job.")
-def get_output(job_id: Optional[str], name: Optional[str]) -> None:
-    """Get the output of a Production Job."""
-    log.warning(
-        "DEPRECATION WARNING: Job outputs is a deprecated feature. This does not work in V2 stack and will be removed completely in the future."
-    )
-    job_controller = JobController()
-    job_controller.retrieve_output(job_name=name, job_id=job_id)
-
-
-@output_cli.command(name="write")
-@click.option(
-    "--file",
-    "-f",
-    required=False,
-    type=click.File("rb"),
-    help="File to read the output from.",
-)
-@click.argument(
-    "input_str", required=False, type=str,
-)
-def write_output(file, input_str) -> None:
-    """Write the output of a Production Job to Anyscale.
-
-    This command should be called from within a Production Job entrypoint. It will record the JSON output of your Job.
-    If you specify neither a file or a string, this command will read from STDIN.
-
-    Examples:
-
-        anyscale job output write -f output.json
-
-        anyscale job output write '{"output": 5}'
-
-        cat output.json | anyscale job output write
-
-    """
-    log.warning(
-        "DEPRECATION WARNING: Job outputs is a deprecated feature. This does not work in V2 stack and will be removed completely in the future."
-    )
-    data = None
-    if input_str and file:
-        raise click.ClickException("Cannot specify both --file and an input string.")
-    if input_str is not None:
-        data = json.loads(input_str)
-    else:
-        data = json.load(file or sys.stdin)
-    anyscale.job.output(data)
