@@ -816,8 +816,8 @@ class ClientBuilder:
             self._log.warning(
                 "DEPRECATION WARNING: Anyscale Connect will be deprecated in future Anyscale versions. "
                 "Please use workspaces: "
-                "https://docs.anyscale.com/user-guide/develop-and-debug/workspaces or Ray Jobs "
-                "https://docs.anyscale.com/user-guide/develop-and-debug/ray-jobs-support instead."
+                "https://docs.anyscale.com/workspaces/get-started or Ray Jobs "
+                "https://docs.anyscale.com/workspaces/interactive-development instead."
             )
 
         _allow_multiple_clients = (
