@@ -821,8 +821,8 @@ class PrepareClusterBlock:
         ).results
         if len(cluster_computes) == 0:
             raise ValueError(
-                f"The cluster compute template {cluster_compute_name}"
-                " is not registered."
+                f"The compute config {cluster_compute_name}"
+                " does not exist or you don't have sufficient permissions."
             )
         return cluster_computes[0].id
 
@@ -908,9 +908,9 @@ class PrepareClusterBlock:
                 ).result
                 if print_warnings:
                     self.log.warning(
-                        f"The cluster is currently using {cluster_compute.name} as the cluster compute, "
+                        f"The cluster is currently using {cluster_compute.name} as the compute config, "
                         f"yet {self.cluster_compute_name} was provided. The cluster will not be updated and will "
-                        f"still use {cluster_compute.name} as the cluster compute. To update the cluster, please specify "
+                        f"still use {cluster_compute.name} as the compute config. To update the cluster, please specify "
                         "`update=True` in the anyscale address, e.g.: "
                         f'`ray.init("anyscale://{cluster_name}", cluster_compute="{self.cluster_compute_name}", update=True")`'
                     )
