@@ -162,7 +162,7 @@ class ProjectBlock:
             ).results
             if len(cluster_computes) == 0:
                 raise click.ClickException(
-                    f"The cluster compute template {cluster_compute_name} does not exist."
+                    f"The compute config {cluster_compute_name} does not exist or you don't have sufficient permissions."
                 )
             parent_cloud_id = cluster_computes[0].config.cloud_id
         elif cluster_compute_dict:
