@@ -53,6 +53,9 @@ HEAD_NODE_TYPE_AWS = "m5.xlarge"  # on demand price ~$0.20 per hour
 HEAD_NODE_TYPE_GCP = "n1-standard-2"  # on demand price ~$0.11 per hour
 CREATE_COMPUTE_CONFIG_TIMEOUT_SECONDS = 300  # 5 minutes
 
+# Workspace verification will fail fast if any of the following logs are found
+WORKSPACE_FAIL_FAST_MATCHING_LOGS = ["Failed to execute"]
+
 
 class CloudFunctionalVerificationType(str, Enum):
     WORKSPACE = "WORKSPACE"
@@ -185,7 +188,7 @@ class CloudFunctionalVerificationController(BaseController):
             time.sleep(delay + jitter)
             delay *= 2  # exponential backoff
         raise ClickException(
-            "Timed out waiting for cluster compute to be created. Please try again."
+            "Timed out waiting for compute config to be created. Please try again."
         )
 
     def _prepare_verification(self, cloud_id: str, cloud_provider: CloudProviders):
@@ -295,6 +298,7 @@ class CloudFunctionalVerificationController(BaseController):
             SessionState.RUNNING,
             SessionState.STARTINGUP,
             SessionState.AWAITINGSTARTUP,
+            SessionState.UPDATING,
         }
 
         with self._create_task(
@@ -398,13 +402,17 @@ class CloudFunctionalVerificationController(BaseController):
             description=f"service for cloud {cloud_id} functional verification",
             project_id=project_id,
             ray_serve_config={
-                "import_path": "serve_hello:entrypoint",
-                "runtime_env": {
-                    "working_dir": "https://github.com/anyscale/docs_examples/archive/refs/heads/main.zip",
-                    "env_vars": {
-                        "SERVE_RESPONSE_MESSAGE": f"cloud functional verification {version}",
-                    },
-                },
+                "applications": [
+                    {
+                        "import_path": "serve_hello:entrypoint",
+                        "runtime_env": {
+                            "working_dir": "https://github.com/anyscale/docs_examples/archive/refs/heads/main.zip",
+                            "env_vars": {
+                                "SERVE_RESPONSE_MESSAGE": f"cloud functional verification {version}",
+                            },
+                        },
+                    }
+                ],
             },
             version=version,
             build_id=cluster_env_build_id,
@@ -588,7 +596,11 @@ class CloudFunctionalVerificationController(BaseController):
             time.sleep(POLL_INTERVAL_SECONDS)
             try:
                 current_status = get_current_status(function.id)
-                self._render_event_log(function_type, function)
+                succeeded = self._render_event_log(function_type, function)
+                if not succeeded:
+                    raise ClickException(
+                        f"{function_type.capitalize()} verification failed! Please check the logs and terminate the {function_type.lower()} manually."
+                    )
             except ClickException as e:
                 raise ClickException(
                     f"Failed to get {function_type.lower()} status: {e}"
@@ -685,20 +697,33 @@ class CloudFunctionalVerificationController(BaseController):
 
     def _format_service_event_log(self, event_log: Any) -> str:
         description = event_log.message.replace("\n", ". ") if event_log.message else ""
-        return f"[bold cyan]{event_log.created_at}[/bold cyan] [bold blue]{event_log.event_type}[/bold blue] [bold green]{description}[/bold green]"
+        if event_log.event_type:
+            return f"[bold cyan]{event_log.created_at}[/bold cyan] [bold blue]{event_log.event_type}[/bold blue] [bold green]{description}[/bold green]"
+        else:
+            return f"[bold cyan]{event_log.created_at}[/bold cyan] [bold green]{description}[/bold green]"
 
     def _render_event_log(
         self, function_type: CloudFunctionalVerificationType, function: Any
-    ):
+    ) -> bool:
         if function_type == CloudFunctionalVerificationType.WORKSPACE:
             workspace_log = self.api_client.get_startup_logs_api_v2_sessions_session_id_startup_logs_get(
                 function.cluster_id, self.event_log_num[function_type], 100000000
             ).result
             if workspace_log.num_lines == self.event_log_num[function_type]:
                 # no new logs
-                return
+                return True
+
+            # Output logs
             self.event_log_tables[function_type].add_row(escape(workspace_log.lines))
             self.event_log_num[function_type] = workspace_log.num_lines
+
+            # fail fast if any of the following logs are found
+            return not any(
+                [
+                    failure_log in workspace_log.lines
+                    for failure_log in WORKSPACE_FAIL_FAST_MATCHING_LOGS
+                ]
+            )
         elif function_type == CloudFunctionalVerificationType.SERVICE:
             service_events = self.api_client.get_service_events_api_v2_services_v2_service_id_events_get(
                 function.id
@@ -709,9 +734,10 @@ class CloudFunctionalVerificationController(BaseController):
                     self._format_service_event_log(service_events[idx])
                 )
             self.event_log_num[function_type] = len(service_events)
+            return True
         else:
             # we should never enter this branch
-            return
+            return False
 
     def get_live_console(
         self, functions_to_verify: List[CloudFunctionalVerificationType]
