@@ -1,4 +1,3 @@
-from datetime import datetime
 import os
 from typing import Any, Dict, Optional, Tuple
 from urllib.parse import urlparse
@@ -51,6 +50,7 @@ from anyscale.util import (
     wait_for_session_start,
 )
 from anyscale.utils.cluster_debug import debug_cluster
+from anyscale.utils.name_utils import gen_valid_name
 
 
 class ClusterController(BaseController):
@@ -578,7 +578,7 @@ class ClusterController(BaseController):
             )
         elif cluster_compute_file:
             cluster_compute_config = self._load_cluster_compute(cluster_compute_file)
-            name = f"cli-config-{datetime.now().isoformat()}"
+            name = gen_valid_name("cli-config")
             cluster_compute = self.anyscale_api_client.create_cluster_compute(
                 CreateClusterCompute(name=name, config=cluster_compute_config)
             ).result
@@ -716,8 +716,8 @@ class ClusterController(BaseController):
             ).result
             self.log.info(
                 f"The cluster is currently using ({current_cluster_compute.name}) (id={current_cluster_compute.id}) as the cluster "
-                f"compute template and {cluster_compute.name} (id={cluster_compute.id}) was provided, so the cluster will "
-                "be updated to use the passed cluster compute template."
+                f"compute config and {cluster_compute.name} (id={cluster_compute.id}) was provided, so the cluster will "
+                "be updated to use the passed cluster compute config."
             )
             needs_start = True
         return needs_start
