@@ -1,14 +1,18 @@
-from datetime import datetime
 import json
 from typing import Any, Dict, IO, List, Optional, Union
 
 import click
 from click import ClickException
-from pydantic import BaseModel, Field, root_validator, validator
 import tabulate
 import yaml
 from yaml.loader import SafeLoader
 
+from anyscale.anyscale_pydantic import (
+    BaseModel,
+    Field,
+    root_validator,
+    validator,
+)
 from anyscale.cli_logger import BlockLogger
 from anyscale.cloud import get_cloud_id_and_name
 from anyscale.cluster_compute import get_cluster_compute_from_name
@@ -32,6 +36,7 @@ from anyscale.sdk.anyscale_client.models.create_cluster_compute_config import (
 from anyscale.sdk.anyscale_client.models.text_query import TextQuery
 from anyscale.util import get_endpoint
 from anyscale.utils.entity_arg_utils import EntityType, IdBasedEntity, NameBasedEntity
+from anyscale.utils.name_utils import gen_valid_name
 
 
 log = BlockLogger()  # Anyscale CLI Logger
@@ -100,6 +105,16 @@ class CreateClusterComputeConfigModel(BaseModel):
         ge=0,
     )
 
+    auto_select_worker_config: Optional[bool] = Field(
+        False,
+        description="If set to true, worker node groups will automatically be selected based on workload.",
+    )
+
+    flags: Optional[Dict[str, Any]] = Field(
+        default_factory=dict,
+        description="A set of advanced flags that can be used to configure a particular workload.",
+    )
+
     @validator("allowed_azs")
     def check_any_option(cls, v: Optional[List[str]]):
         if not v:
@@ -158,17 +173,33 @@ class ComputeConfigController(BaseController):
 
         Information in output: Link to cluster compute in UI, cluster compute id
         """
+        enable_compute_config_versioning = self.api_client.check_is_feature_flag_on_api_v2_userinfo_check_is_feature_flag_on_get(
+            "compute-config-versioning"
+        ).result.is_on
+
+        if not enable_compute_config_versioning and name and ":" in name:
+            log.warning(
+                "The compute config's name contains colons (`:`). "
+                "Colons will be a reserved character after 04/01/2024, "
+                "and before then, we are discouraging using colons in the name of compute configs. "
+                "Also, all existing colons will be replaced by hyphens at 04/01/2024 "
+                "as the versioning feature is enabled."
+            )
 
         try:
             cluster_compute: Dict[str, Any] = yaml.load(
                 cluster_compute_file, Loader=SafeLoader
             )
         except Exception as e:  # noqa: BLE001
-            raise ClickException(f"Could not load cluster compute file: {e}")
+            raise ClickException(f"Could not load compute config file: {e}")
 
         cluster_compute_config_model = CreateClusterComputeConfigModel(
             **cluster_compute
         )
+        if cluster_compute_config_model.max_workers is not None:
+            log.warning(
+                "Warning: global `max_workers` is deprecated for Anyscale Ray 2.7+. Please use global resource max instead: https://docs.anyscale.com/configure/compute-configs/global-resource-min-max"
+            )
         cluster_compute_config = CreateClusterComputeConfig(
             cloud_id=cluster_compute_config_model.cloud_id,
             max_workers=cluster_compute_config_model.max_workers,
@@ -180,19 +211,34 @@ class ComputeConfigController(BaseController):
             gcp_advanced_configurations_json=cluster_compute_config_model.gcp,
             maximum_uptime_minutes=cluster_compute_config_model.maximum_uptime_minutes,
             idle_termination_minutes=cluster_compute_config_model.idle_termination_minutes,
+            auto_select_worker_config=cluster_compute_config_model.auto_select_worker_config,
+            flags=cluster_compute_config_model.flags,
         )
         if name is None:
-            name = f"cli-config-{datetime.now().isoformat()}"
+            name = gen_valid_name("cli-config")
+
         cluster_compute_response = self.anyscale_api_client.create_cluster_compute(
-            CreateClusterCompute(name=name, config=cluster_compute_config)
+            CreateClusterCompute(
+                name=name,
+                config=cluster_compute_config,
+                new_version=enable_compute_config_versioning,
+            )
         )
         created_cluster_compute = cluster_compute_response.result
         cluster_compute_id = created_cluster_compute.id
         cluster_compute_name = created_cluster_compute.name
+        cluster_compute_version = created_cluster_compute.version
         url = get_endpoint(f"/configurations/cluster-computes/{cluster_compute_id}")
-        log.info(f"View this cluster compute at: {url}")
-        log.info(f"Cluster compute id: {cluster_compute_id}")
-        log.info(f"Cluster compute name: {cluster_compute_name}")
+        if enable_compute_config_versioning and cluster_compute_version > 1:
+            log.info(f"A new version of {cluster_compute_name} was created.")
+        else:
+            log.info("A new compute config was created.")
+
+        log.info(f"View this compute config at: {url}.")
+        log.info(f"Compute config id: {cluster_compute_id}.")
+        log.info(f"Compute config name: {cluster_compute_name}.")
+        if enable_compute_config_versioning:
+            log.info(f"Compute config version: {cluster_compute_version}.")
 
     def delete(
         self, cluster_compute_name: Optional[str], id: Optional[str]  # noqa: A002
@@ -205,7 +251,7 @@ class ComputeConfigController(BaseController):
 
         if int(bool(cluster_compute_name)) + int(bool(id)) != 1:
             raise ClickException(
-                "Not deleted. Please provide exactly one of: cluster compute name, id."
+                "Not deleted. Please provide exactly one of: compute config name, id."
             )
 
         if id:
@@ -220,11 +266,11 @@ class ComputeConfigController(BaseController):
             cluster_results = query_response.results
             if len(cluster_results) != 1:
                 raise ClickException(
-                    f"Not deleted. No cluster compute template exists with the name {cluster_compute_name}."
+                    f"Not deleted. No compute config exists with the name {cluster_compute_name}."
                 )
             cluster_compute_id = cluster_results[0].id
         self.anyscale_api_client.delete_cluster_compute(cluster_compute_id)
-        log.info("Cluster compute deleted.")
+        log.info("Compute config deleted.")
 
     def archive(
         self, compute_config_entity: Union[IdBasedEntity, NameBasedEntity]
@@ -316,7 +362,7 @@ class ComputeConfigController(BaseController):
             headers=["ID", "NAME", "CLOUD", "LAST MODIFIED AT", "URL"],
             tablefmt="plain",
         )
-        print(f"Cluster computes:\n{table}")
+        print(f"Compute configs:\n{table}")
 
     def get(
         self,
@@ -339,5 +385,8 @@ class ComputeConfigController(BaseController):
             cluster_compute_id
         ).result.config
 
-        formatted_config_json = json.dumps(compute_config.to_dict(), indent=2)
+        compute_config_dict = {
+            key: val for key, val in compute_config.to_dict().items() if val is not None
+        }
+        formatted_config_json = json.dumps(compute_config_dict, indent=2)
         print(formatted_config_json)
