@@ -1,34 +1,19 @@
 from abc import ABC, abstractmethod
 import asyncio
-import contextlib
 from datetime import datetime, timedelta
-import json
 import os
 import random
 import string
 import time
-import traceback
-from typing import Any, Callable, cast, Dict, List, Optional, Tuple, Union
+from typing import Any, cast, Dict, List, Optional, Tuple, Union
 
+import aiohttp
 import click
-from pydantic import BaseModel
-import requests
-from requests.adapters import HTTPAdapter
 import tabulate
-from urllib3 import Retry
 import yaml
 
-from anyscale.api_utils.exceptions.log_retrieval_errors import (
-    ExpectedLogRetrievalError,
-    wrap_as_unsupported_log_retrieval_method_error,
-    wrap_job_run_log_not_retrievable_on_active_cluster_error,
-)
-from anyscale.api_utils.job_logs_util import (
-    _get_job_logs_from_storage_bucket,
-    _get_job_logs_from_storage_bucket_streaming,
-    _get_logs_from_active_job_run,
-)
-from anyscale.api_utils.logs_util import _stream_log_from_ray_websocket
+from anyscale.anyscale_pydantic import BaseModel
+from anyscale.api_utils.job_logs_util import _get_job_logs_from_storage_bucket_streaming
 from anyscale.cli_logger import LogsLogger
 from anyscale.client.openapi_client import (
     ComputeTemplate,
@@ -42,6 +27,7 @@ from anyscale.client.openapi_client.models.decorated_production_job import (
     DecoratedProductionJob,
 )
 from anyscale.client.openapi_client.models.ha_job_states import HaJobStates
+from anyscale.client.openapi_client.models.job_queue_config import JobQueueConfig
 from anyscale.controllers.base_controller import BaseController
 from anyscale.models.job_model import JobConfig
 from anyscale.project import infer_project_id
@@ -55,10 +41,9 @@ from anyscale.sdk.anyscale_client.models.sort_by_clause_jobs_sort_field import (
 from anyscale.sdk.anyscale_client.models.sort_order import SortOrder
 from anyscale.util import (
     get_endpoint,
-    handle_and_log_lines_in_click_exception,
     is_anyscale_workspace,
     poll,
-    populate_dict_with_workspace_config_if_exists,
+    populate_unspecified_cluster_configs_from_current_workspace,
     validate_job_config_dict,
 )
 from anyscale.utils.connect_helpers import search_entities
@@ -94,7 +79,7 @@ class LogProvider(ABC):
         pass
 
     @abstractmethod
-    def query(
+    async def query(
         self, start_timestamp_ns: int, end_timestamp_ns: int
     ) -> List[Tuple[int, str]]:
         """Query logs for a time range, return list of (timestamp_ns, log_line)."""
@@ -111,35 +96,52 @@ class LogProvider(ABC):
 
 
 class LokiLogProvider(LogProvider):
-    def __init__(self, dns_name: str, anyscale_token: str, query: str) -> None:
+    def __init__(
+        self,
+        dns_name: str,
+        token: str,
+        query: str,
+        limit: int,
+        tenant_id: str,
+        direction: str,
+    ) -> None:
         # https://grafana.com/docs/loki/latest/api/#get-lokiapiv1query_range
         self.url = f"https://{dns_name}/loki/api/v1/query_range"
         self.params: Dict[str, Union[str, int]] = {
-            "token": anyscale_token,
             "query": query,
-            "limit": 1000,
+            "limit": limit,
             "start": 0,
             "end": int(time.time() * 1e9),
-            "direction": "forward",
+            "direction": direction,
         }
+        self.tenant_id = tenant_id
+        self.token = token
+        self.headers = {
+            "X-Scope-OrgID": self.tenant_id,
+            "Authorization": f"Bearer {self.token}",
+        }
+        self.ingress_auth_max_retry = 10
+        self.ingress_auth_retry_wait_time_s = 0.1
+        # We are retrying 401 here because there could be a race condition
+        # between pinging Loki and the auth token getting persisted and
+        # accessible by the ingress. In that case the status code will be
+        # 401 and thus need to retry.
+        # Also retrying 500 for possible transient errors.
+        self.ingress_auth_retry_status = {401, 500}
+        self.loki_query_max_retry_time_s = 60
 
-    def __enter__(self, *args, **kwargs):
-        self.session = requests.Session()
-        self.session.mount(
-            "https://",
-            HTTPAdapter(
-                max_retries=Retry(total=10, backoff_factor=0.1, allowed_methods=["GET"])
-            ),
-        )
+    async def __aenter__(self, *args, **kwargs):
+        self.session = aiohttp.ClientSession()
+        self.ingress_auth_current_retry_count = 0
         return self
 
-    def close(self) -> None:
-        self.session.close()
+    async def close(self):
+        await self.session.close()
 
-    def __exit__(self, *args, **kwargs):
-        self.close()
+    async def __aexit__(self, *args, **kwargs):
+        await self.close()
 
-    def query(
+    async def query(
         self, start_timestamp_ns: int, end_timestamp_ns: int
     ) -> List[Tuple[int, str]]:
         if start_timestamp_ns == 0:
@@ -150,20 +152,39 @@ class LokiLogProvider(LogProvider):
         self.params["start"] = start_timestamp_ns
         self.params["end"] = end_timestamp_ns
 
-        resp = self.session.get(self.url, params=self.params)
-        resp.raise_for_status()
-
-        resp_data = resp.json()
-        if resp_data["status"] != "success":
-            raise click.ClickException("Querying Anyscale log endpoint failed.")
-        if resp_data["data"]["resultType"] != "streams":
-            raise click.ClickException("Invalid metrics type.")
-
+        query_start_time = time.time()
         lines = []
-        for log_chunk in resp_data["data"]["result"]:
-            # TODO(simon): potentially annotate the log with the log_chunk["stream"] metadata
-            for timestamp, line in log_chunk["values"]:
-                lines.append((int(timestamp), line))
+        while (
+            self.ingress_auth_current_retry_count < self.ingress_auth_max_retry
+            and time.time() - query_start_time < self.loki_query_max_retry_time_s
+        ):
+            async with self.session.get(
+                self.url,
+                params=self.params,
+                headers=self.headers,
+                timeout=self.loki_query_max_retry_time_s,
+            ) as resp:
+                if resp.status in self.ingress_auth_retry_status:
+                    self.ingress_auth_current_retry_count += 1
+                    await asyncio.sleep(self.ingress_auth_retry_wait_time_s)
+                    if (
+                        self.ingress_auth_current_retry_count
+                        < self.ingress_auth_max_retry
+                    ):
+                        continue
+
+                resp.raise_for_status()
+                resp_data = await resp.json()
+                if resp_data["status"] != "success":
+                    raise click.ClickException("Querying Anyscale log endpoint failed.")
+                if resp_data["data"]["resultType"] != "streams":
+                    raise click.ClickException("Invalid metrics type.")
+
+                for log_chunk in resp_data["data"]["result"]:
+                    # TODO(simon): potentially annotate the log with the log_chunk["stream"] metadata
+                    for timestamp, line in log_chunk["values"]:
+                        lines.append((int(timestamp), line))
+                return lines
 
         return lines
 
@@ -230,11 +251,12 @@ class JobController(BaseController):
             "description": description,
             "workspace_id": workspace_id,
         }
-        config_dict = populate_dict_with_workspace_config_if_exists(
-            config_dict, self.anyscale_api_client
+
+        config_dict = populate_unspecified_cluster_configs_from_current_workspace(
+            config_dict, self.anyscale_api_client,
         )
-        job_config = JobConfig.parse_obj(config_dict)
-        return job_config
+
+        return JobConfig.parse_obj(config_dict)
 
     def generate_config_from_file(
         self,
@@ -246,7 +268,7 @@ class JobController(BaseController):
         config_dict = self._load_config_dict_from_file(job_config_file)
         config_dict["workspace_id"] = workspace_id
         validate_job_config_dict(config_dict, self.api_client)
-        config_dict = populate_dict_with_workspace_config_if_exists(
+        config_dict = populate_unspecified_cluster_configs_from_current_workspace(
             config_dict, self.anyscale_api_client
         )
         job_config = JobConfig.parse_obj(config_dict)
@@ -288,6 +310,12 @@ class JobController(BaseController):
             max_retries=job_config.max_retries,
         )
 
+        job_queue_config = (
+            JobQueueConfig(**job_config.job_queue_config)
+            if job_config.job_queue_config
+            else None
+        )
+
         job = self.api_client.create_job_api_v2_decorated_ha_jobs_create_post(
             CreateInternalProductionJob(
                 name=job_config.name or self._generate_random_job_name(),
@@ -295,6 +323,7 @@ class JobController(BaseController):
                 project_id=project_id,
                 workspace_id=job_config.workspace_id,
                 config=config_object,
+                job_queue_config=job_queue_config,
             )
         ).result
         self.log.info(
@@ -602,67 +631,6 @@ class JobController(BaseController):
                 job_state = job.state.current_state
         return job
 
-    def _print_logs_for_active_job_run(
-        self, job_run_id: str, should_follow: bool
-    ) -> None:
-        """
-        Prints logs from the active cluster the job is run on.
-        """
-        with handle_and_log_lines_in_click_exception(log_callback=self.log.error):
-            if should_follow:
-                # Use websockets (to continuously poll logs)
-                with wrap_as_unsupported_log_retrieval_method_error():
-                    websocket_url: str = self.api_client.get_job_logs_stream_api_v2_logs_job_logs_stream_job_id_get(
-                        job_id=job_run_id
-                    ).result.stream_url
-
-                with wrap_job_run_log_not_retrievable_on_active_cluster_error(
-                    job_run_id, raise_connection_issue_as_cli_error=True
-                ):
-                    # (run_until_complete() is used here b/c the KeyboardInterrupt exception is
-                    # not propogated all the way up (and likely being handled by internals of asyncio.run()))
-                    asyncio.get_event_loop().run_until_complete(
-                        _stream_log_from_ray_websocket(
-                            websocket_url,
-                            log_callback=self.log.log,
-                            remove_escape_chars=False,
-                        )
-                    )
-            else:
-                # Make a one-time request to get logs
-                logs = asyncio.run(
-                    _get_logs_from_active_job_run(
-                        self.anyscale_api_client,
-                        job_run_id,
-                        raise_connection_issue_as_cli_error=True,
-                        remove_escape_chars=False,
-                    )
-                )
-                for line in logs.splitlines():
-                    self.log.log(line)
-
-    def _print_logs_for_inactive_job_run(self, job_run_id: str) -> None:
-        """Retrieves logs from the cloud storage bucket for a job run in terminal state"""
-        logs = ""
-        with self.log.spinner("Retrieving logs..."):
-            # For jobs on a cluster that is terminating, the logs are in-flight between cluster and storage bucket
-            # of the cloud. In that case, wait for a bit and try again.
-            for _ in poll(interval_secs=5, max_iter=10):
-                logs = asyncio.run(
-                    _get_job_logs_from_storage_bucket(
-                        self.anyscale_api_client,
-                        job_run_id=job_run_id,
-                        remove_escape_chars=False,
-                    )
-                )
-                if logs:
-                    break
-        if not logs:
-            self.log.error("Logs are not available for this job run.")
-            return
-        for line in logs.splitlines():
-            self.log.log(line)
-
     def _print_logs_using_streaming_job_logs(
         self, job_run_id: str, follow: bool = False
     ) -> None:
@@ -739,20 +707,11 @@ class JobController(BaseController):
         should_follow: bool = False,
         all_attempts: bool = False,
     ) -> None:
-        """Uses fallback logic to print logs for a (production) job to terminal output.
-
-        * If the job is running, it will retrieve logs from the cluster the job is run on.
-        * If the job is not running, it will retrieve logs from Anyscale's backend cache.
-        * If the logs no longer live in the cache, or the cloud the job is on is using Anyscale V2,
-            it falls back to retrieving logs from the storage bucket of the cloud.
         """
-        # This logic will be simplified once the migration to Anyscale V2 is complete.
+        Fetches logs for a production job using the streaming log S3 bucket.
+        """
         job = self._wait_for_a_job_run(job_id, job_name)
-        is_streaming_job_logs_on = self._api_client.check_is_feature_flag_on_api_v2_userinfo_check_is_feature_flag_on_get(
-            "ray-job-log-streaming"
-        ).result.is_on
         last_job_run_id: str = job.last_job_run_id
-        job_state: HaJobStates = job.state.current_state
         job_run_ids: List[str] = (
             [last_job_run_id]
             if not all_attempts
@@ -762,60 +721,9 @@ class JobController(BaseController):
             self.log.open_block(
                 job_run_id, f"Job Run Id: {job_run_id}", auto_close=True,
             )
-            if is_streaming_job_logs_on:
-                # New path for retrieving logs. Works regardless if cluster is running or dead.
-                self._print_logs_using_streaming_job_logs(
-                    job_run_id, follow=should_follow and job_run_id == last_job_run_id
-                )
-            else:
-                # 1. Last job run, and 2. job is RUNNING implies (==) job run is RUNNING
-                is_last_job_run_and_active = (
-                    job_run_id == last_job_run_id and job_state == HaJobStates.RUNNING
-                )
-
-                get_logs_fns: List[Callable[[], None]] = [
-                    *(
-                        # If the job run is RUNNING, retrieve logs from the active cluster
-                        []
-                        if not is_last_job_run_and_active
-                        else [
-                            lambda: self._print_logs_for_active_job_run(
-                                job_run_id, should_follow  # noqa: B023
-                            )
-                        ]
-                    ),
-                    # If the previous retrieval method failed or the job is not running
-                    # retrieve logs from Anyscale's cache
-                    lambda: self._print_logs_for_inactive_job_run(
-                        job_run_id  # noqa: B023
-                    ),
-                ]
-                for fn in get_logs_fns:
-                    with contextlib.suppress(ExpectedLogRetrievalError):
-                        fn()
-                        # If `fn` didn't raise `ExpectedLogRetrievalError`, we assume logs were successfully printed.
-                        # In this case, break out of this loop and log for the next `job_run_id`
-                        break
-
-    def retrieve_output(
-        self, job_name: Optional[str] = None, job_id: Optional[str] = None,
-    ) -> None:
-        if not job_id:
-            job = self._resolve_job_object(job_id, job_name)
-            job_id = job.id
-
-        try:
-            output = self.api_client.find_structured_job_output_api_v2_structured_outputs_ha_job_output_ha_job_id_get(
-                ha_job_id=job_id
+            self._print_logs_using_streaming_job_logs(
+                job_run_id, follow=should_follow and job_run_id == last_job_run_id
             )
-            self.log.log(json.dumps(output.result.output, indent=2))
-        except Exception as e:  # noqa: BLE001
-            traceback.print_exc()
-            # Swallow if pretty print logic fails
-            with contextlib.suppress(Exception):
-                j = json.loads(e.body)  # type: ignore
-                detail = j.get("error", {}).get("detail")
-                self.log.error(detail)
 
     def _generate_random_job_name(self) -> str:
         """Generates a random job name
