@@ -23,13 +23,19 @@ class MachineController(BaseController):
         self.console = Console()
 
     def list_machines(
-        self, cloud: Optional[str], cloud_id: Optional[str]
+        self,
+        cloud: Optional[str],
+        cloud_id: Optional[str],
+        machine_pool_name: Optional[str],
     ) -> ListMachinesResponse:
-        if cloud is not None:
-            cloud_id, _ = get_cloud_id_and_name(self.api_client, cloud_name=cloud)
+        if cloud is not None or cloud_id is not None:
+            if cloud is not None:
+                cloud_id, _ = get_cloud_id_and_name(self.api_client, cloud_name=cloud)
+        elif machine_pool_name is not None:
+            machine_pool_name = machine_pool_name
 
         response: ListMachinesResponse = self.api_client.list_machines_api_v2_machines_get(
-            cloud_id
+            cloud_id=cloud_id, machine_pool_name=machine_pool_name,
         ).result
         return response
 
