@@ -1,46 +1,84 @@
 import os
-from typing import List, Optional
+from typing import Any, Dict, List, Optional
 
 import click
 import tabulate
 import yaml
 
+from anyscale import AnyscaleSDK
 from anyscale.cli_logger import BlockLogger
-from anyscale.client.openapi_client import ComputeTemplate, ProductionJob
+
+# There are two API clients: one that's exposed to customers via the SDK (ExternalApi)
+# and one that's only used internally by the UI and CLI (InternalApi).
+from anyscale.client.openapi_client.api.default_api import DefaultApi as InternalApi
 from anyscale.client.openapi_client.models.decorated_list_service_api_model import (
     DecoratedListServiceAPIModel,
 )
 from anyscale.client.openapi_client.models.decorated_production_service_v2_api_model import (
     DecoratedProductionServiceV2APIModel,
 )
-from anyscale.client.openapi_client.models.ha_job_states import HaJobStates
 from anyscale.controllers.base_controller import BaseController
-from anyscale.formatters.service_formatter import format_service_config
 from anyscale.models.service_model import ServiceConfig
 from anyscale.project import infer_project_id
+from anyscale.sdk.anyscale_client.models import (
+    ApplyServiceModel,
+    RollbackServiceModel,
+    ServiceModel,
+)
 from anyscale.tables import ServicesTable
 from anyscale.util import (
     get_endpoint,
-    poll,
-    populate_dict_with_workspace_config_if_exists,
+    populate_unspecified_cluster_configs_from_current_workspace,
 )
 from anyscale.utils.runtime_env import override_runtime_env_config
 from anyscale.utils.workload_types import Workload
-
-
-_POLL_SERVICE_VERSION_INFO_INTERVAL_IN_SECS = 3.0
-_POLL_SERVICE_VERSION_INFO_TIMEOUT_IN_SECS = 60.0 * 3
+from anyscale.utils.workspace_notification import (
+    send_workspace_notification,
+    WorkspaceNotification,
+    WorkspaceNotificationAction,
+)
 
 
 class ServiceController(BaseController):
-    def __init__(
-        self, log: Optional[BlockLogger] = None, initialize_auth_api_client: bool = True
-    ):
-        if log is None:
-            log = BlockLogger()
+    def __init__(self, *, sdk: Optional[AnyscaleSDK] = None):
+        super().__init__()
+        self.log = BlockLogger()
+        self.sdk = sdk if sdk is not None else AnyscaleSDK()
 
-        super().__init__(initialize_auth_api_client=initialize_auth_api_client)
-        self.log = log
+    @property
+    def internal_api_client(self) -> InternalApi:
+        return self.api_client
+
+    def _format_apply_service_model(self, config: ServiceConfig) -> ApplyServiceModel:
+        if not config.ray_serve_config:
+            raise click.ClickException(
+                "ray_serve_config must be provided in the service configuration."
+            )
+
+        if "import_path" in config.ray_serve_config:
+            self.log.warning(
+                "The single application Ray Serve config is deprecated in Ray 2.8 and "
+                "removed in Ray 2.9. Please migrate to the multi-application Ray Serve "
+                "config format. Please see "
+                "https://docs.ray.io/en/latest/serve/multi-app.html#migrating-from-a-single-application-config "
+                "for migration instructions."
+            )
+
+        return ApplyServiceModel(
+            name=config.name,
+            description=config.description or "Service updated from CLI",
+            project_id=config.project_id,
+            version=config.version,
+            canary_percent=config.canary_percent,
+            ray_serve_config=config.ray_serve_config,
+            ray_gcs_external_storage_config=config.ray_gcs_external_storage_config,
+            build_id=config.build_id,
+            compute_config_id=config.compute_config_id,
+            rollout_strategy=config.rollout_strategy,
+            config=config.config,
+            auto_complete_rollout=config.auto_complete_rollout,
+            max_surge_percent=config.max_surge_percent,
+        )
 
     def _get_services_by_name(
         self,
@@ -56,17 +94,19 @@ class ServiceController(BaseController):
         """
         creator_id = None
         if created_by_me:
-            user_info_response = self.api_client.get_user_info_api_v2_userinfo_get()
+            user_info_response = (
+                self.internal_api_client.get_user_info_api_v2_userinfo_get()
+            )
             creator_id = user_info_response.result.id
 
         services_list = []
-        resp = self.api_client.list_services_api_v2_services_v2_get(
+        resp = self.internal_api_client.list_services_api_v2_services_v2_get(
             project_id=project_id, name=name, creator_id=creator_id, count=10,
         )
         services_list.extend(resp.results)
         paging_token = resp.metadata.next_paging_token
         while paging_token is not None and len(services_list) < max_items:
-            resp = self.api_client.list_services_api_v2_services_v2_get(
+            resp = self.internal_api_client.list_services_api_v2_services_v2_get(
                 project_id=project_id,
                 name=name,
                 creator_id=creator_id,
@@ -137,8 +177,8 @@ class ServiceController(BaseController):
                 )
             service_id = self._get_service_id_from_name(service_name, project_id)
         elif service_config_file is not None:
-            service_config: ServiceConfig = self._read_service_config_file(
-                service_config_file
+            service_config: ServiceConfig = self.parse_service_config_dict(
+                self.read_service_config_file(service_config_file)
             )
             # Allow the passed project_id to override the one in the file.
             project_id = project_id or service_config.project_id
@@ -150,26 +190,25 @@ class ServiceController(BaseController):
 
         return service_id
 
-    def generate_config_from_file(
+    def override_config_options(
         self,
-        service_config_file,
+        service_config: ServiceConfig,
         *,
+        auto_complete_rollout: bool,
         name: Optional[str] = None,
         description: Optional[str] = None,
         version: Optional[str] = None,
+        max_surge_percent: Optional[int] = None,
         canary_percent: Optional[int] = None,
         rollout_strategy: Optional[str] = None,
     ) -> ServiceConfig:
         """
-        Given the input file path and user overrides, this method
+        Given the input config and user overrides, this method
         constructs the expected service configuration.
 
         Note that if the command is run from a workspace, some values
         may be overwritten.
         """
-        service_config: ServiceConfig = self._read_service_config_file(
-            service_config_file
-        )
         if name:
             service_config.name = name
 
@@ -188,60 +227,63 @@ class ServiceController(BaseController):
         if service_config.rollout_strategy:
             service_config.rollout_strategy = service_config.rollout_strategy.upper()
 
+        if auto_complete_rollout is not None:
+            service_config.auto_complete_rollout = auto_complete_rollout
+
+        if max_surge_percent is not None:
+            service_config.max_surge_percent = max_surge_percent
+
+        if (
+            service_config.max_surge_percent is not None
+            and service_config.rollout_strategy == "IN_PLACE"
+        ):
+            raise ValueError(
+                "--max-surge-percent is not supported for IN_PLACE rollouts."
+            )
+
         return service_config
 
-    def _get_maximum_uptime_minutes(self, service: ProductionJob) -> Optional[int]:
-        compute_config: ComputeTemplate = self.api_client.get_compute_template_api_v2_compute_templates_template_id_get(
-            service.config.compute_config_id
-        ).result
-        return compute_config.config.maximum_uptime_minutes
-
-    def _get_maximum_uptime_output(self, maximum_uptime_minutes: Optional[int]) -> str:
-        if maximum_uptime_minutes and maximum_uptime_minutes > 0:
-            return f"set to {maximum_uptime_minutes} minutes"
-        return "disabled"
-
-    def _get_additional_log_if_maximum_uptime_enabled(
-        self, maximum_uptime_minutes: Optional[int]
-    ) -> str:
-        if maximum_uptime_minutes and maximum_uptime_minutes > 0:
-            return " This may cause disruptions. To disable, update the compute config."
-        return ""
-
-    def _read_service_config_file(self, service_config_file: str) -> ServiceConfig:
+    def read_service_config_file(self, service_config_file: str) -> Dict[str, Any]:
         if not os.path.exists(service_config_file):
             raise click.ClickException(f"Config file {service_config_file} not found.")
 
         with open(service_config_file) as f:
-            config_dict = yaml.safe_load(f)
+            return yaml.safe_load(f)
 
-        updated_config_dict = populate_dict_with_workspace_config_if_exists(
-            config_dict, self.anyscale_api_client
+    def parse_service_config_dict(self, config_dict: Dict[str, Any]) -> ServiceConfig:
+        # If running in a workspace, auto-populate unspecified fields.
+        config_dict = populate_unspecified_cluster_configs_from_current_workspace(
+            config_dict, self.sdk, populate_name=True
         )
-        return ServiceConfig.parse_obj(updated_config_dict)
+        return ServiceConfig.parse_obj(config_dict)
 
-    def rollout(
+    def rollout(  # noqa: PLR0913
         self,
-        service_config_file: str,
+        config_dict: Dict[str, Any],
+        *,
+        auto_complete_rollout: bool,
         name: Optional[str] = None,
         version: Optional[str] = None,
+        max_surge_percent: Optional[int] = None,
         canary_percent: Optional[int] = None,
         rollout_strategy: Optional[str] = None,
     ):
         """
         Deploys a Service 2.0.
         """
-        config = self.generate_config_from_file(
-            service_config_file,
+        config = self.override_config_options(
+            self.parse_service_config_dict(config_dict),
             name=name,
             version=version,
+            max_surge_percent=max_surge_percent,
             canary_percent=canary_percent,
             rollout_strategy=rollout_strategy,
+            auto_complete_rollout=auto_complete_rollout,
         )
 
         config.project_id = infer_project_id(
-            self.anyscale_api_client,
-            self.api_client,
+            self.sdk,
+            self.internal_api_client,
             self.log,
             project_id=config.project_id,
             cluster_compute_id=config.compute_config_id,
@@ -267,26 +309,31 @@ class ServiceController(BaseController):
 
         self._overwrite_runtime_env_in_v2_ray_serve_config(config)
 
-        service_v2_config = format_service_config(config)
-
-        service = self.api_client.apply_service_api_v2_services_v2_apply_put(
-            service_v2_config
-        ).result
-
-        current_state = service.goal_state
-        self.log.info(
-            f"Service {service.id} has been deployed. Service is transitioning towards: {current_state}."
+        apply_service_model: ApplyServiceModel = self._format_apply_service_model(
+            config
         )
+        service: ServiceModel = self.sdk.rollout_service(apply_service_model).result
+        self.log.info(f"Service {service.id} rollout initiated.")
         self.log.info(
             f'View the service in the UI at {get_endpoint(f"/services/{service.id}")}'
         )
 
-        if config.canary_percent is not None and len(service.versions) > 1:
-            if config.canary_percent == 100:
-                self.log.warning(
-                    "The canary percent has been set to 100%. This will complete the rollout."
-                )
-            self._log_canary_version_query(service)
+        url = service.base_url
+        auth_token = service.auth_token
+        self.log.info(
+            "You can query the service endpoint using the curl request below:"
+        )
+        self.log.info(f"curl -H 'Authorization: Bearer {auth_token}' {url}")
+
+        send_workspace_notification(
+            self.sdk,
+            WorkspaceNotification(
+                body=f"Service {service.name} rollout initiated.",
+                action=WorkspaceNotificationAction(
+                    type="navigate-service", title="View Service", value=service.id,
+                ),
+            ),
+        )
 
     def _overwrite_runtime_env_in_v2_ray_serve_config(self, config: ServiceConfig):
         """Modifies config in place."""
@@ -295,8 +342,8 @@ class ServiceController(BaseController):
             for ray_serve_app_config in ray_serve_config["applications"]:
                 ray_serve_app_config["runtime_env"] = override_runtime_env_config(
                     runtime_env=ray_serve_app_config.get("runtime_env"),
-                    anyscale_api_client=self.anyscale_api_client,
-                    api_client=self.api_client,
+                    anyscale_api_client=self.sdk,
+                    api_client=self.internal_api_client,
                     workload_type=Workload.SERVICES,
                     compute_config_id=config.compute_config_id,
                     log=self.log,
@@ -305,65 +352,13 @@ class ServiceController(BaseController):
         else:
             ray_serve_config["runtime_env"] = override_runtime_env_config(
                 runtime_env=ray_serve_config.get("runtime_env"),
-                anyscale_api_client=self.anyscale_api_client,
-                api_client=self.api_client,
+                anyscale_api_client=self.sdk,
+                api_client=self.internal_api_client,
                 workload_type=Workload.SERVICES,
                 compute_config_id=config.compute_config_id,
                 log=self.log,
             )
 
-    def _log_canary_version_query(self, service: DecoratedProductionServiceV2APIModel):
-        """
-        This method logs the canary version's query endpoint.
-
-        Since the url and token from the service object are not initially available,
-        we poll until they become available.
-        """
-        assert len(service.versions) > 1
-        # We assume versions are returned in order of creation.
-        canary_version = service.versions[1]
-        production_job_id = (
-            canary_version.production_job_ids[0]
-            if canary_version.production_job_ids
-            else None
-        )
-        if production_job_id:
-            canary_version_query_printed = False
-            for _ in poll(
-                interval_secs=_POLL_SERVICE_VERSION_INFO_INTERVAL_IN_SECS,
-                timeout_secs=_POLL_SERVICE_VERSION_INFO_TIMEOUT_IN_SECS,
-            ):
-                try:
-                    service_v1 = self.api_client.get_job_api_v2_decorated_ha_jobs_production_job_id_get(
-                        production_job_id
-                    ).result
-                    url = service_v1.url
-                    token = service_v1.token
-                    service_version_current_state = service_v1.state.current_state
-                    if url and token:
-                        self.log.info()
-                        self.log.info(
-                            "You can query the service endpoint using the curl request below:"
-                        )
-                        self.log.info(f"curl -H 'Authorization: Bearer {token}' {url}")
-
-                        if service_version_current_state in {
-                            HaJobStates.PENDING,
-                            HaJobStates.AWAITING_CLUSTER_START,
-                        }:
-                            self.log.info(
-                                "Please note the canary version is still pending and may take several minutes to begin successfully returning requests."
-                            )
-                        canary_version_query_printed = True
-                        break
-                except Exception:  # noqa: BLE001
-                    # We keep re-retrying until the timeout
-                    pass
-            if not canary_version_query_printed:
-                self.log.info(
-                    "Please refer to the Service details page if you would like to query the canary version."
-                )
-
     def list(
         self,
         *,
@@ -393,7 +388,7 @@ class ServiceController(BaseController):
         services = []
         if service_id:
             service_v2_result: DecoratedProductionServiceV2APIModel = (
-                self.api_client.get_service_api_v2_services_v2_service_id_get(
+                self.internal_api_client.get_service_api_v2_services_v2_service_id_get(
                     service_id
                 ).result
             )
@@ -433,15 +428,12 @@ class ServiceController(BaseController):
         self.log.info(f'View your Services in the UI at {get_endpoint("/services")}')
         self.log.info(f"Services:\n{table}")
 
-    def archive(self, service_id: str):
-        raise click.ClickException(
-            f"Archive {service_id} is not currently supported for v2 services."
-            "Please contact Anyscale support for more information."
-        )
-
-    def rollback(self, service_id: str) -> None:
-        service = self.api_client.rollback_service_api_v2_services_v2_service_id_rollback_post(
-            service_id
+    def rollback(self, service_id: str, max_surge_percent: Optional[int] = None):
+        service: ServiceModel = self.sdk.rollback_service(
+            service_id,
+            rollback_service_model=RollbackServiceModel(
+                max_surge_percent=max_surge_percent
+            ),
         ).result
 
         self.log.info(f"Service {service.id} rollback initiated.")
@@ -449,11 +441,9 @@ class ServiceController(BaseController):
             f'View the service in the UI at {get_endpoint(f"/services/{service.id}")}'
         )
 
-    def terminate(self, service_id: str) -> None:
-        self.api_client.terminate_service_api_v2_services_v2_service_id_terminate_post(
-            service_id
-        )
-        self.log.info(f"Service {service_id} terminate initiated.")
+    def terminate(self, service_id: str):
+        service: ServiceModel = self.sdk.terminate_service(service_id).result
+        self.log.info(f"Service {service.id} terminate initiated.")
         self.log.info(
-            f'View the service in the UI at {get_endpoint(f"/services/{service_id}")}'
+            f'View the service in the UI at {get_endpoint(f"/services/{service.id}")}'
         )
