@@ -13,7 +13,7 @@ from anyscale.controllers.base_controller import BaseController
 from anyscale.feature_flags import FLAG_DEFAULT_WORKING_DIR_FOR_PROJ
 from anyscale.util import get_endpoint
 from anyscale.utils.workspace_utils import (
-    extract_workspace_parameters_from_cluster_config,
+    extract_workspace_parameters,
     WorkspaceCommandContext,
 )
 from anyscale.workspace import load_workspace_id_or_throw, write_workspace_id_to_disk
@@ -88,7 +88,7 @@ class WorkspaceController(BaseController):
         ).result
         if self.api_client.check_is_feature_flag_on_api_v2_userinfo_check_is_feature_flag_on_get(
             FLAG_DEFAULT_WORKING_DIR_FOR_PROJ
-        ):
+        ).result.is_on:
             return project.directory_name
         else:
             return project.name
@@ -99,7 +99,7 @@ class WorkspaceController(BaseController):
         cluster_id = workspace.cluster_id
         project_id = workspace.project_id
 
-        return extract_workspace_parameters_from_cluster_config(
+        return extract_workspace_parameters(
             cluster_id,
             project_id,
             api_client=self.api_client,
