@@ -1,6 +1,6 @@
 import ipaddress
 import re
-from typing import Iterable, List, Tuple
+from typing import Iterable, List, Optional, Tuple
 
 import click
 from click import ClickException
@@ -19,7 +19,7 @@ from anyscale.client.openapi_client.models import (
     CloudAnalyticsEventCloudResource,
     CreateCloudResourceGCP,
 )
-from anyscale.shared_anyscale_utils.conf import ANYSCALE_HOST
+from anyscale.shared_anyscale_utils.conf import ANYSCALE_CORS_ORIGIN
 from anyscale.utils.cloud_utils import CloudSetupError
 from anyscale.utils.gcp_managed_setup_utils import enable_project_apis
 from anyscale.utils.gcp_utils import (
@@ -65,7 +65,7 @@ def verify_gcp_networking(
             project=project_id, network=vpc_name
         )
     except NotFound:
-        logger.log_resource_not_found_error("VPC", vpc_name)
+        logger.log_resource_not_found_error("VPC", vpc_name, project_id)
         return False
 
     subnet_name = resources.gcp_subnet_ids[
@@ -82,7 +82,7 @@ def verify_gcp_networking(
             project=project_id, subnetwork=subnet_name, region=cloud_region,
         )
     except NotFound:
-        logger.log_resource_not_found_error("Subnet", subnet_name)
+        logger.log_resource_not_found_error("Subnet", subnet_name, project_id)
         return False
 
     if subnet.network != vpc.self_link:
@@ -93,8 +93,8 @@ def verify_gcp_networking(
         logger.internal.error(f"Subnet {subnet_name} is not part of {vpc_name}!")
         return False
 
-    if is_private_service_cloud and not _vpc_has_proxy_only_subnet(
-        factory, project_id, cloud_region, vpc
+    if is_private_service_cloud and not _get_proxy_only_subnet_in_vpc(
+        factory, project_id, cloud_region, vpc_name
     ):
         logger.internal.warning(
             f"Could not find a subnet with purpose `{PROXY_ONLY_SUBNET_PURPOSE}` in the VPC {vpc_name}. A proxy-only subnet is required to run private Anyscale Services in your account. See https://cloud.google.com/load-balancing/docs/proxy-only-subnets for more details."
@@ -116,23 +116,21 @@ def _gcp_subnet_has_enough_capacity(
     return False
 
 
-def _vpc_has_proxy_only_subnet(
+def _get_proxy_only_subnet_in_vpc(
     factory: GoogleCloudClientFactory,
     project_id: str,
     cloud_region: str,
-    vpc: compute_v1.types.compute.Network,
-) -> bool:
+    vpc_name: str,
+) -> Optional[compute_v1.types.compute.Subnetwork]:
     # Check if there exists a subnet in the VPC and region with purpose `REGIONAL_MANAGED_PROXY`
     subnets = factory.compute_v1.SubnetworksClient().list(
         project=project_id, region=cloud_region
     )
     for subnet in subnets:
-        if (
-            subnet.network == vpc.self_link
-            and subnet.purpose == PROXY_ONLY_SUBNET_PURPOSE
-        ):
-            return True
-    return False
+        subnet_vpc = subnet.network.split("/")[-1]
+        if vpc_name == subnet_vpc and subnet.purpose == PROXY_ONLY_SUBNET_PURPOSE:
+            return subnet
+    return None
 
 
 def verify_gcp_project(  # noqa: PLR0911
@@ -343,11 +341,13 @@ def verify_gcp_dataplane_service_account(
     return True
 
 
-def verify_firewall_policy(  # noqa: PLR0911, PLR0912
+def verify_firewall_policy(  # noqa: PLR0911, PLR0912, C901, PLR0913
     factory: GoogleCloudClientFactory,
     resources: CreateCloudResourceGCP,
     project_id: str,
     cloud_region: str,
+    use_shared_vpc: bool,
+    is_private_service_cloud: bool,
     logger: GCPLogger,
     strict: bool = False,
 ) -> bool:
@@ -378,7 +378,9 @@ def verify_firewall_policy(  # noqa: PLR0911, PLR0912
                 project=project_id, firewall_policy=firewall_policy, region=cloud_region
             )
         except NotFound:
-            logger.log_resource_not_found_error("Firewall Policy", firewall_policy)
+            logger.log_resource_not_found_error(
+                "Firewall Policy", firewall_policy, project_id
+            )
             return False
 
     if not any(
@@ -403,7 +405,11 @@ def verify_firewall_policy(  # noqa: PLR0911, PLR0912
 
     rules = _firewall_rules_from_proto_resp(firewall.rules)
     if not check_inbound_firewall_permissions(
-        rules, Protocol.tcp, {22}, ipaddress.ip_network("0.0.0.0/0")
+        rules,
+        Protocol.tcp,
+        {22},
+        ipaddress.ip_network("0.0.0.0/0"),
+        check_ingress_source_range=False,
     ):
         logger.internal.warning(
             "Firewall policy does not allow inbound access on port 22. Certain features may be unavailable."
@@ -411,7 +417,11 @@ def verify_firewall_policy(  # noqa: PLR0911, PLR0912
         if strict:
             return False
     if not check_inbound_firewall_permissions(
-        rules, Protocol.tcp, {443}, ipaddress.ip_network("0.0.0.0/0")
+        rules,
+        Protocol.tcp,
+        {443},
+        ipaddress.ip_network("0.0.0.0/0"),
+        check_ingress_source_range=False,
     ):
         logger.internal.warning(
             "Firewall policy does not allow inbound access on port 443. Accessing features like Jupyter & Ray Dashboard may not work."
@@ -419,6 +429,47 @@ def verify_firewall_policy(  # noqa: PLR0911, PLR0912
         if strict:
             return False
 
+    # If shared VPC is used, we need to check the firewall rule for Anyscale services is added
+    # Since our service account cannot update the firewall policy in host project, so we need to make sure the rule is added during cloud setup
+    if use_shared_vpc and not (
+        check_inbound_firewall_permissions(
+            rules, Protocol.tcp, {8000}, ipaddress.ip_network("35.191.0.0/16")
+        )
+        and check_inbound_firewall_permissions(
+            rules, Protocol.tcp, {8000}, ipaddress.ip_network("130.211.0.0/22")
+        )
+    ):
+        logger.internal.warning(
+            "Firewall policy does not allow inbound access on port 8000 from '35.191.0.0/16' and '130.211.0.0/22'. "
+            "This is required for Anyscale services to work correctly. "
+            "See https://cloud.google.com/load-balancing/docs/health-checks#fw-rule for more details."
+        )
+        if strict:
+            return False
+
+    if use_shared_vpc and is_private_service_cloud:
+        proxy_only_subnet = _get_proxy_only_subnet_in_vpc(
+            factory, project_id, cloud_region, vpc_name
+        )
+        if not proxy_only_subnet:
+            logger.internal.warning(
+                f"Could not find a subnet with purpose `{PROXY_ONLY_SUBNET_PURPOSE}` in the VPC {vpc_name}. A proxy-only subnet is required to run private Anyscale Services in your account. See https://cloud.google.com/load-balancing/docs/proxy-only-subnets for more details."
+            )
+            if strict:
+                return False
+        elif not check_inbound_firewall_permissions(
+            rules,
+            Protocol.tcp,
+            {8000},
+            ipaddress.ip_network(proxy_only_subnet.ip_cidr_range),
+        ):
+            logger.internal.warning(
+                "Firewall policy does not allow inbound access from the proxy-only subnet on port 8000. "
+                "This is required for private Anyscale services to work correctly. "
+            )
+            if strict:
+                return False
+
     if not check_inbound_firewall_permissions(rules, Protocol.tcp, None, subnet):
         logger.internal.error(
             "Firewall policy does not allow for internal communication, see https://cloud.google.com/vpc/docs/using-firewalls#common-use-cases-allow-internal for how to configure such a rule."
@@ -558,7 +609,7 @@ def verify_cloud_storage(  # noqa: PLR0911, PLR0912
 
     if not any(
         (
-            ANYSCALE_HOST in cors_config.get("origin")
+            ANYSCALE_CORS_ORIGIN in cors_config.get("origin")
             and "*" in cors_config.get("responseHeader", [])
             and "GET" in cors_config.get("method", [])
         )
@@ -566,7 +617,7 @@ def verify_cloud_storage(  # noqa: PLR0911, PLR0912
     ):
         logger.internal.warning(
             f"Bucket {bucket_name} does not have the correct CORS rule for Anyscale. This is safe to ignore if you are not using Anyscale UI.\n"
-            "If you are using the UI, please create the correct CORS rule for Anyscale according to https://docs.anyscale.com/user-guide/onboard/clouds/deploy-on-gcp#appendix-detailed-resource-requirements"
+            "If you are using the UI, please create the correct CORS rule for Anyscale according to https://docs.anyscale.com/cloud-deployment/gcp/deploy-cloud?cloud-deployment=custom#4-create-an-anyscale-cloud"
         )
         if strict:
             return False
