@@ -4,9 +4,15 @@ from typing import Any, Dict, List, Optional, Union
 from urllib.parse import urlparse
 
 import click
-from pydantic import BaseModel, Extra, Field, root_validator, validator
 import yaml
 
+from anyscale.anyscale_pydantic import (
+    BaseModel,
+    Extra,
+    Field,
+    root_validator,
+    validator,
+)
 from anyscale.cli_logger import LogsLogger
 from anyscale.cluster_compute import (
     get_cluster_compute_from_name,
@@ -307,7 +313,12 @@ class BaseHAJobConfig(BaseModel):
         None,
         description="The compute config version filter that you want to use. Only applicable if the compute_config was specified by name. "
         "Versions are positive integers. "
-        "Deprecated behavior: Setting version to None is equivalent to setting version to '1'.",
+        "Setting version to None is equivalent to setting version to latest version.",
+    )
+
+    job_queue_config: Optional[Dict[str, Any]] = Field(
+        None,
+        description="Configuration specifying semantic of the execution using job queues",
     )
 
     @validator("compute_config_version")
