@@ -425,7 +425,7 @@ def get_default_project(
         if not parent_cloud_id:
             raise click.ClickException(
                 "Please specify a cloud for this command either through the cloud or "
-                "cluster compute arguments."
+                "compute config arguments."
             )
         default_project = anyscale_api_client.get_default_project(
             parent_cloud_id=parent_cloud_id
