@@ -85,11 +85,18 @@ def _upload_and_rewrite_working_dir_ray_serve_config(
     if not ray_serve_config:
         return ray_serve_config
     res = copy.deepcopy(ray_serve_config)
-    if isinstance(ray_serve_config, dict):
-        runtime_env = ray_serve_config.get("runtime_env")
-        if runtime_env is not None:
-            new_runtime_env = upload_and_rewrite_working_dir(runtime_env)
-            res["runtime_env"] = new_runtime_env
+    if isinstance(res, dict):
+        if "applications" in res:
+            for ray_serve_app_config in res["applications"]:
+                runtime_env = ray_serve_app_config.get("runtime_env")
+                if runtime_env is not None:
+                    new_runtime_env = upload_and_rewrite_working_dir(runtime_env)
+                    ray_serve_app_config["runtime_env"] = new_runtime_env
+        else:
+            runtime_env = res.get("runtime_env")
+            if runtime_env is not None:
+                new_runtime_env = upload_and_rewrite_working_dir(runtime_env)
+                res["runtime_env"] = new_runtime_env
     return res
 
 
@@ -135,6 +142,7 @@ class AnyscaleSDK(DefaultApi):  # type: ignore
         # Adds base path "v0" for API endpoints
         endpoint_url = host.rstrip("/") + "/v0"
         configuration = anyscale_client.Configuration(host=endpoint_url)
+        configuration.proxy = os.environ.get("https_proxy")
         configuration.connection_pool_maxsize = 100
         if auth_token is None:
             auth_token, _ = AuthenticationBlock._load_credentials()
