@@ -35,7 +35,7 @@ CONFIGS_TO_REPLICATE = [
     ".tmux.conf",
 ]
 
-# Default cluster storage dir.
+# Default cluster storage directory.
 CLUSTER_STORAGE_DIR = "/mnt/cluster_storage"
 
 # Default VSCode server dir.
@@ -496,11 +496,11 @@ def find_base_snapshot_from_workspace(
 
 def find_base_snapshot() -> Optional[str]:  # noqa: PLR0911
     """
-        Find the base snapshot for the job/service/workspace used to create current workspace.
+    Find the base snapshot for the job/service/workspace used to create current workspace.
 
-        Returns:
-            If exists, return the EFS path or URI of the base snapshot.
-            otherwise, return None.
+    Returns:
+        If exists, return the EFS path or URI of the base snapshot.
+        otherwise, return None.
     """
     if not BASE_SNAPSHOT:
         return None
@@ -635,7 +635,7 @@ def _split_s3_path(s3_path: str) -> Tuple[str, str]:
 
 
 def checkpoint_job(job_id, runtime_env: Dict[str, Any]) -> Dict[str, Any]:
-    """ Deprecated in v2 stack: we are moving away from EFS and towards remote buckets.
+    """Deprecated in v2 stack: we are moving away from EFS and towards remote buckets.
 
     Checkpoint the runtime environment and working directory of a job.
 
@@ -656,10 +656,13 @@ def checkpoint_job(job_id, runtime_env: Dict[str, Any]) -> Dict[str, Any]:
         # We need a custom path for s3 urls because urllib.request.urlretrieve doesn't work for s3 urls
         if runtime_env["working_dir"].startswith("s3://"):
             import boto3
+            import botocore.config
 
             bucket, key = _split_s3_path(runtime_env["working_dir"])
             # boto3 client initialized inside clusters already have AWS credentials set
-            s3_client = boto3.client("s3")
+            s3_client = boto3.client(
+                "s3", config=botocore.config.Config(signature_version="s3v4")
+            )
             s3_client.download_file(bucket, key, working_dir)
         elif runtime_env["working_dir"].startswith("gs://"):
             # checkpoint_job is deprecated in v2 stack,
