@@ -47,7 +47,9 @@ from anyscale.client.openapi_client.models.user_info import UserInfo
 from anyscale.cluster_compute import get_cluster_compute_from_name
 import anyscale.conf
 from anyscale.conf import MINIMUM_RAY_VERSION
+from anyscale.feature_flags import FLAG_DEFAULT_WORKING_DIR_FOR_PROJ
 from anyscale.sdk.anyscale_client.api.default_api import DefaultApi as BaseApi
+from anyscale.sdk.anyscale_client.models.cluster import Cluster
 from anyscale.sdk.anyscale_client.models.compute_template import ComputeTemplate
 import anyscale.shared_anyscale_utils.conf as shared_anyscale_conf
 from anyscale.utils.cloud_utils import CloudSetupError
@@ -71,7 +73,7 @@ DEFAULT_RAY_VERSION = "1.7.0"
 # The V2 stack has some delay before changing the state of a new cluster from
 # Terminated to StartingUp. Allow the terminated state for the duration of this
 # grace period before erroring.
-TERMINATED_STATE_GRACE_PERIOD_SECONDS = 60
+TERMINATED_STATE_GRACE_PERIOD_SECONDS = 5 * 60  # 5 minutes
 
 MEMORY_DB_OUTPUT = """  MemoryDB:
     Description: MemoryDB cluster
@@ -248,8 +250,13 @@ def _client(name: str, region: str) -> Any:
     return _resource(name, region).meta.client
 
 
-def _get_role(role_name: str, region: str) -> Optional[Boto3Resource]:
-    iam = _resource("iam", region)
+def _get_role(
+    role_name: str, region: str, boto3_session: Optional[boto3.Session] = None
+) -> Optional[Boto3Resource]:
+    if boto3_session is None:
+        iam = _resource("iam", region)
+    else:
+        iam = boto3_session.resource("iam")
     role = iam.Role(role_name)  # type: ignore
     try:
         role.load()
@@ -342,17 +349,23 @@ def _get_memorydb_cluster_config(
         raise e
 
 
-def get_available_regions() -> List[str]:
+def get_available_regions(boto3_session: Optional[boto3.Session] = None) -> List[str]:
+    if boto3_session is None:
+        boto3_session = boto3.Session()
     try:
-        client = boto3.client("ec2")
+        client = boto3_session.client("ec2")
     except NoRegionError:
         # If there is no region, default to `us-west-2`
-        client = boto3.client("ec2", region_name="us-west-2")
+        client = boto3_session.client("ec2", region_name="us-west-2")
     return [region["RegionName"] for region in client.describe_regions()["Regions"]]
 
 
-def get_availability_zones(region: str) -> List[str]:
-    client = boto3.client("ec2", region_name=region)
+def get_availability_zones(
+    region: str, boto3_session: Optional[boto3.Session] = None
+) -> List[str]:
+    if boto3_session is None:
+        boto3_session = boto3.Session()
+    client = boto3_session.client("ec2", region_name=region)
     return [
         region["ZoneName"]
         for region in client.describe_availability_zones()["AvailabilityZones"]
@@ -365,21 +378,19 @@ def get_project_directory_name(project_id: str, api_client: ProductApi = None) -
 
     # TODO (yiran): return error early if project doesn't exist.
     resp = api_client.get_project_api_v2_projects_project_id_get(project_id)
-    directory_name = resp.result.directory_name
+    if api_client.check_is_feature_flag_on_api_v2_userinfo_check_is_feature_flag_on_get(
+        FLAG_DEFAULT_WORKING_DIR_FOR_PROJ
+    ).result.is_on:
+        directory_name = resp.result.directory_name
+    else:
+        directory_name = resp.result.name
     assert len(directory_name) > 0, "Empty directory name found."
     return cast(str, directory_name)
 
 
-def get_working_dir(
-    cluster_config: Dict[str, Any], project_id: str, api_client: ProductApi = None
-) -> str:
-    working_dir: Optional[str] = (
-        cluster_config.get("metadata", {}).get("anyscale", {}).get("working_dir")
-    )
-    if working_dir:
-        return working_dir
-    else:
-        return f"/home/ray/{get_project_directory_name(project_id, api_client)}"
+def get_working_dir(project_id: str, api_client: ProductApi = None) -> str:
+    project_directory_name = get_project_directory_name(project_id, api_client)
+    return f"/home/ray/{project_directory_name}"
 
 
 def get_wheel_url(
@@ -767,11 +778,20 @@ def prepare_cloudformation_template(
     cfn_stack_name: str,
     cloud_id: str,
     enable_head_node_fault_tolerance: bool,
+    boto3_session: Optional[boto3.Session] = None,
+    is_anyscale_hosted: bool = False,
 ) -> str:
-    with open(f"{anyscale.conf.ROOT_DIR_PATH}/anyscale-cloud-setup.yaml") as f:
-        body = f.read()
+    if is_anyscale_hosted:
+        with open(f"{anyscale.conf.ROOT_DIR_PATH}/anyscale-cloud-setup-oa.yaml") as f:
+            body = f.read()
+    else:
+        with open(f"{anyscale.conf.ROOT_DIR_PATH}/anyscale-cloud-setup.yaml") as f:
+            body = f.read()
+        body = body.replace(
+            "$ALLOWED_ORIGIN", shared_anyscale_conf.ANYSCALE_CORS_ORIGIN
+        )
 
-    azs = get_availability_zones(region)
+    azs = get_availability_zones(region, boto3_session)
     subnet_templates: List[str] = []
     subnets_route_table_association: List[str] = []
     subnets_with_availability_zones: List[str] = []
@@ -832,7 +852,6 @@ def prepare_cloudformation_template(
         "$SUBNETS_WITH_AVAILABILITY_ZONES",
         f'[{",".join(subnets_with_availability_zones)}]',
     )
-    body = body.replace("$ALLOWED_ORIGIN", shared_anyscale_conf.ANYSCALE_HOST)
     body = body.replace("$EFSMountTargets", "\n".join(efs_mount_targets))
 
     iam_policy_parameters = [
@@ -933,36 +952,74 @@ def validate_job_config_dict(
 
     if compute_config and compute_config.archived_at:
         raise click.ClickException(
-            "This job is using an archived compute config. To submit this job, specify a new compute config."
+            "This job is using an archived compute config. To submit this job, use another compute config."
         )
 
 
-def populate_dict_with_workspace_config_if_exists(
-    config_dict: Dict[str, Any], anyscale_api_client: BaseApi
-) -> Dict[str, Any]:
+def get_cluster_model_for_current_workspace(
+    anyscale_api_client: BaseApi,
+) -> Optional[Cluster]:
+    """If run within a workspace, returns the `Cluster` info for that workspace.
+
+    Else, returns `None`.
     """
-    If the service/job is submitted from the workspace, we populate values from the workspace
-    if the config doesn't specify them, such as cluster env and compute config.
+    session_id = os.environ.get("ANYSCALE_SESSION_ID")
+    if not session_id or not is_anyscale_workspace():
+        return None
+
+    return anyscale_api_client.get_cluster(session_id).result
+
+
+def populate_unspecified_cluster_configs(
+    config: Dict[str, Any], workspace_cluster: Cluster, *, populate_name: bool = False
+):
+    """Populates unspecified fields in the config from a workspace cluster model.
+
+    This is used to fill in smart defaults when deploying from a workspace.
+
+    Fields that are defaulted:
+        - `project_id` if it is not specified.
+        - `build_id` if none of {`build_id, `cluster_env`} are specified.
+        - `compute_config_id` if none of {`cloud`, `compute_config`, `compute_config_id`} are specified.
+        - `name` if `populate_name` is passed and it is not specified.
+
+    The defaulted name will be the name of the wor
     """
+    if "project_id" not in config and "project" not in config:
+        config["project_id"] = workspace_cluster.project_id
+
+    if "build_id" not in config and "cluster_env" not in config:
+        config["build_id"] = workspace_cluster.cluster_environment_build_id
+
     if (
-        "ANYSCALE_EXPERIMENTAL_WORKSPACE_ID" in os.environ
-        and "ANYSCALE_SESSION_ID" in os.environ
+        "compute_config" not in config
+        and "compute_config_id" not in config
+        and "cloud" not in config
     ):
-        cluster = anyscale_api_client.get_cluster(
-            os.environ["ANYSCALE_SESSION_ID"]
-        ).result
-        # If the service/job configs are not specified, infer them from the workspace:
-        if "build_id" not in config_dict and "cluster_env" not in config_dict:
-            config_dict["build_id"] = cluster.cluster_environment_build_id
-        if "project_id" not in config_dict and "project" not in config_dict:
-            config_dict["project_id"] = cluster.project_id
-        if (
-            "compute_config" not in config_dict
-            and "compute_config_id" not in config_dict
-            and "cloud" not in config_dict
-        ):
-            config_dict["compute_config_id"] = cluster.cluster_compute_id
-    return config_dict
+        config["compute_config_id"] = workspace_cluster.cluster_compute_id
+
+    if populate_name and "name" not in config:
+        name = workspace_cluster.name
+        # All workspace cluster names should start with this prefix.
+        # Defensively default to the name as-is if they don't.
+        if name.startswith("workspace-cluster-"):
+            name = name[len("workspace-cluster-") :]
+
+        config["name"] = name
+
+    return config
+
+
+def populate_unspecified_cluster_configs_from_current_workspace(
+    config: Dict[str, Any], anyscale_api_client: BaseApi, *, populate_name: bool = False
+) -> Dict[str, Any]:
+    workspace_cluster = get_cluster_model_for_current_workspace(anyscale_api_client)
+    if workspace_cluster is not None:
+        config = populate_unspecified_cluster_configs(
+            config, workspace_cluster, populate_name=populate_name
+        )
+
+    return config
 
 
 def filter_actions_from_policy_document(
@@ -1065,14 +1122,3 @@ T = TypeVar("T")
 
 def _coerce_to_list(maybe_list: Union[T, List[T]]) -> List[T]:
     return maybe_list if isinstance(maybe_list, list) else [maybe_list]
-
-
-@contextmanager
-def handle_and_log_lines_in_click_exception(
-    log_callback: Callable[[str], Any]
-) -> Generator[None, None, None]:
-    try:
-        yield
-    except click.ClickException as e:
-        for line in e.message.splitlines():
-            log_callback(line)
