@@ -6,9 +6,9 @@ from typing import Dict, List, Optional, Set, Tuple
 
 from botocore.exceptions import ClientError, NoCredentialsError
 from click import ClickException
-from pydantic import BaseModel
 
 from anyscale import __version__ as anyscale_version
+from anyscale.anyscale_pydantic import BaseModel
 from anyscale.aws_iam_policies import (
     ANYSCALE_IAM_POLICIES,
     AnyscaleIAMPolicy,
