@@ -2,8 +2,7 @@ from dataclasses import dataclass
 import inspect
 from typing import Any, Callable, Dict, Generic, List, Optional, TypeVar
 
-from pydantic import BaseModel
-
+from anyscale.anyscale_pydantic import BaseModel
 from anyscale.client.openapi_client.models.session import Session
 from anyscale.sdk.anyscale_client.models.cluster import Cluster
 from anyscale.sdk.anyscale_client.sdk import AnyscaleSDK
