@@ -1,7 +1,7 @@
 import os
 from string import Template
 import time
-from typing import Any, Dict, Optional
+from typing import Any, Dict, List, Optional
 
 from click import ClickException
 from google.api_core.exceptions import BadRequest, Forbidden, NotFound, PermissionDenied
@@ -13,9 +13,9 @@ from google.cloud.compute_v1.types import (
 from google.iam.v1.iam_policy_pb2 import SetIamPolicyRequest
 from google.iam.v1.policy_pb2 import Binding
 from googleapiclient.errors import HttpError
-from pydantic import BaseModel
 import yaml
 
+from anyscale.anyscale_pydantic import BaseModel
 from anyscale.cli_logger import CloudSetupLogger
 from anyscale.client.openapi_client.models import CloudAnalyticsEventCloudResource
 import anyscale.conf
@@ -155,7 +155,7 @@ def get_deployment_resources(
     deployment_name: str,
     project_id: str,
     anyscale_access_service_account_name: str,
-):
+) -> Dict[str, str]:
     """
     Get the resources in a deployment.
     """
@@ -318,6 +318,7 @@ def generate_deployment_manager_config(  # noqa: PLR0913
 
     params = {
         "ANYSCALE_HOST": shared_anyscale_conf.ANYSCALE_HOST,
+        "ANYSCALE_CORS_ORIGIN": shared_anyscale_conf.ANYSCALE_CORS_ORIGIN,
         "REGION": region,
         # `cloud_id` is used as the deployment name, which cannot contain underscores.
         "CLOUD_ID": cloud_id.replace("_", "-").lower(),
@@ -576,6 +577,122 @@ def remove_firewall_policy_associations(
             )
 
 
+def delete_gcp_tls_certificates(
+    factory: GoogleCloudClientFactory, project_id: str, cloud_id: str
+):
+
+    # Initialize the Compute Engine client
+    certificate_manager_client = (
+        factory.certificate_manager_v1.CertificateManagerClient()
+    )
+
+    try:
+        location_path = certificate_manager_client.common_location_path(
+            project_id, "global"
+        )
+    except NotFound:
+        # no certificate
+        return
+    except Exception as e:  # noqa: BLE001
+        raise ClickException(f"Failed to delete tls certificate. {e}")
+
+    try:
+        certificate_objects = certificate_manager_client.list_certificates(
+            parent=location_path
+        )
+
+        certificate_maps_objects = certificate_manager_client.list_certificate_maps(
+            parent=location_path
+        )
+    except Exception as e:  # noqa: BLE001
+        raise ClickException(f"Failed to delete tls certificate. {e}")
+
+    certificates = list(certificate_objects)
+    certificate_maps = list(certificate_maps_objects)
+
+    certificates = filter_resources(certificates, cloud_id)
+    certificate_maps = filter_resources(certificate_maps, cloud_id)
+
+    all_certificate_map_entries = []
+    for certificate in certificates:
+        certificate_name = certificate.split("/")[-1]
+        if certificate_name:
+            try:
+                certificate_map_entry_path = certificate_manager_client.certificate_map_path(
+                    project_id, "global", certificate_name
+                )
+                certificate_map_entry_objects = certificate_manager_client.list_certificate_map_entries(
+                    parent=certificate_map_entry_path
+                )
+            except NotFound:
+                # no certificate_map
+                continue
+            except Exception as e:  # noqa: BLE001
+                raise ClickException(f"Failed to delete tls certificate. {e}")
+
+            certificate_map_entries = list(certificate_map_entry_objects)
+            all_certificate_map_entries.extend(certificate_map_entries)
+
+    all_certificate_map_entries = filter_resources(
+        all_certificate_map_entries, cloud_id
+    )
+
+    wait_on_operation(
+        [
+            certificate_manager_client.delete_certificate_map_entry(
+                name=certificate_map_entry
+            )
+            for certificate_map_entry in all_certificate_map_entries
+        ]
+    )
+
+    wait_on_operation(
+        [
+            certificate_manager_client.delete_certificate(name=certificate)
+            for certificate in certificates
+        ]
+    )
+
+    wait_on_operation(
+        [
+            certificate_manager_client.delete_certificate_map(name=certificate_map)
+            for certificate_map in certificate_maps
+        ]
+    )
+
+
+def wait_on_operation(operations):
+    """
+    Waits until all operations are finished before returning
+    """
+    for operation in operations:
+        try:
+            operation.result()
+        except Exception as e:  # noqa: BLE001
+            raise ClickException(f"Failed to wait for operation to complete. {e}")
+
+
+def filter_resources(resources, cloud_id) -> List[str]:
+    ret = []
+    for resource in resources:
+        resource_labels = resource.labels
+        if (
+            "anyscale-cloud-id" in resource_labels
+            and resource_labels["anyscale-cloud-id"] == cloud_id
+        ):
+            ret.append(resource.name)
+    return ret
+
+
+def filter_resources_on_cloud(resources, cloud_id):
+    ret = []
+    for resource in resources:
+        if cloud_id in resource.name:
+            ret.append(resource)
+
+    return ret
+
+
 def get_or_create_memorystore_gcp(
     factory: GoogleCloudClientFactory,
     cloud_id: str,
