@@ -134,15 +134,18 @@ def check_inbound_firewall_permissions(
     protocol: Protocol,
     ports: Optional[Set[int]],
     source_range: IPNetwork,
+    check_ingress_source_range: bool = True,
 ) -> bool:
     ports_remaining = ports.copy() if ports is not None else set(range(1, 65536))
     for rule in rules:
+
         if (
             rule.direction != Direction.INGRESS
             or not isinstance(source_range, type(rule.network))
             or not source_range.subnet_of(rule.network)  # type: ignore
-        ):
+        ) and check_ingress_source_range:
             continue
+
         if rule.protocol == Protocol.all or protocol == rule.protocol:
             ports_remaining.difference_update(rule.ports or {})
 
