@@ -80,9 +80,10 @@ def cloud_test_data() -> Cloud:
         credentials="credentials",
         creator_id="creator_id",
         type="PUBLIC",
-        created_at=datetime.now(timezone.utc),
+        created_at=datetime(2023, 10, 14),
         config='{"max_stopped_instances": 0}',
         state="ACTIVE",
+        is_default=False,
     )
 
 
@@ -260,6 +261,8 @@ def userinfo_test_data() -> UserInfo:
                 public_identifier="mock_org_1",
                 default_cloud_id="mock_default_cloud_id",
                 sso_required=False,
+                is_general_platform_enabled=True,
+                is_private_endpoints_enabled=False,
             )
         ],
         ld_hash="mock_ld_hash",
