@@ -398,6 +398,7 @@ def test_build_cluster_env_if_needed(
         assert observed_result == cluster_env_name
 
 
+@pytest.mark.parametrize("cluster_compute_version", [None, 2])
 @pytest.mark.parametrize("cluster_compute_name", [None, "test_cluster_compute_name"])
 @pytest.mark.parametrize("cluster_compute_dict", [None, {"cloud_id": "mock_cloud_id"}])
 @pytest.mark.parametrize("cloud_name", [None, "test_cloud_name"])
@@ -406,6 +407,7 @@ def test_get_cluster_compute_id(
     cluster_compute_name: Optional[str],
     cluster_compute_dict: Optional[Dict[str, str]],
     cloud_name: Optional[str],
+    cluster_compute_version: Optional[int],
 ):
     cloud_id = "test_cloud_id"
     mock_get_selected_cloud_id_or_default = Mock(return_value=cloud_id)
@@ -429,12 +431,16 @@ def test_get_cluster_compute_id(
         get_selected_cloud_id_or_default=mock_get_selected_cloud_id_or_default,
     ):
         observed_result = mock_prepare_cluster_block._get_cluster_compute_id(
-            "test_project_id", cluster_compute_name, cluster_compute_dict, cloud_name
+            "test_project_id",
+            cluster_compute_name,
+            cluster_compute_dict,
+            cloud_name,
+            cluster_compute_version,
         )
 
     if cluster_compute_name:
         mock_prepare_cluster_block._get_cluster_compute_id_from_name.assert_called_once_with(
-            "test_project_id", cluster_compute_name
+            cluster_compute_name, cluster_compute_version
         )
         assert observed_result == "mock_cluster_compute_template"
     else:
@@ -451,9 +457,12 @@ def test_get_cluster_compute_id(
         assert observed_result == "mock_registered_template"
 
 
+@pytest.mark.parametrize("cluster_compute_version", [None, 2])
 @pytest.mark.parametrize("cluster_computes", [[], [Mock(id="test_cluster_compute_id")]])
 def test_get_cluster_compute_id_from_name(
-    mock_prepare_cluster_block, cluster_computes: List[Any]
+    mock_prepare_cluster_block,
+    cluster_computes: List[Any],
+    cluster_compute_version: Optional[int],
 ):
     mock_prepare_cluster_block.api_client.search_compute_templates_api_v2_compute_templates_search_post = Mock(
         return_value=Mock(results=cluster_computes)
@@ -461,12 +470,12 @@ def test_get_cluster_compute_id_from_name(
     if len(cluster_computes) == 0:
         with pytest.raises(ValueError):
             mock_prepare_cluster_block._get_cluster_compute_id_from_name(
-                "test_project_id", "test_cluster_compute_name"
+                "test_cluster_compute_name", cluster_compute_version
             )
     else:
         assert (
             mock_prepare_cluster_block._get_cluster_compute_id_from_name(
-                "test_project_id", "test_cluster_compute_name"
+                "test_cluster_compute_name", cluster_compute_version
             )
             == cluster_computes[0].id
         )
@@ -516,6 +525,7 @@ def test_validate_new_cluster_compute_and_env_match_existing_cluster(
     mock_prepare_cluster_block.cluster_compute_dict = "test_cluster_compute_dict"
     mock_prepare_cluster_block.cloud_name = "test_cloud_name"
     mock_prepare_cluster_block.cluster_env_dict = "test_cluster_env_dict"
+    mock_prepare_cluster_block.cluster_compute_version = 2
     mock_prepare_cluster_block._get_cluster_compute_id = Mock()
 
     if build_matches:
@@ -546,6 +556,7 @@ def test_validate_new_cluster_compute_and_env_match_existing_cluster(
         assert observed_result
 
 
+@pytest.mark.parametrize("cluster_compute_version", [0, 2])
 @pytest.mark.parametrize(
     "get_prepare_cluster_action",
     [
@@ -560,6 +571,7 @@ def test_init(
     get_prepare_cluster_action: bool,
     cluster_compute_name: Optional[str],
     cluster_env_name: Optional[str],
+    cluster_compute_version: Optional[int],
 ):
     cluster_name = "test_cluster_name"
     mock_get_prepare_cluster_action = Mock(return_value=get_prepare_cluster_action)
@@ -612,6 +624,7 @@ def test_init(
             cluster_env_dict=mock_cluster_env_dict,
             cluster_env_revision=mock_cluster_env_revision,
             ray=Mock(),
+            cluster_compute_version=cluster_compute_version,
         )
         prepare_cluster_block.prepare()
         assert prepare_cluster_block.cluster_name == "test_cluster_name"
@@ -645,6 +658,7 @@ def test_init(
                 cluster_compute_name,
                 mock_cluster_compute_dict,
                 "test_cloud_name",
+                cluster_compute_version,
             )
         mock_wait_for_app_build.assert_called_once_with("test_project_id", build_id)
         mock_start_or_create_cluster.assert_called_once_with(
