@@ -4,6 +4,7 @@ import tempfile
 from typing import Any, Callable, Optional
 from unittest.mock import Mock, patch
 
+import click
 from click import ClickException
 import pytest
 import yaml
@@ -62,6 +63,46 @@ def test_get(
         cluster_env_controller.api_client.get_build_api_v2_builds_build_id_get.assert_not_called()
 
 
+@pytest.mark.parametrize("cluster_env_name", ["mock_cluster_env_name", None])
+@pytest.mark.parametrize("cluster_env_id", ["mock_cluster_env_id", None])
+def test_archive(
+    mock_auth_api_client, cluster_env_name: Optional[str], cluster_env_id: Optional[str]
+) -> None:
+    cluster_env_controller = ClusterEnvController()
+    cluster_env_controller.anyscale_api_client.get_cluster_environment = Mock(
+        return_value=Mock(result=Mock(name="mock_cluster_env_name"))
+    )
+    mock_get_cluster_env_from_name = Mock(return_value=Mock(id="mock_cluster_env_id"))
+    should_throw_error = bool(cluster_env_name) == bool(cluster_env_id)
+
+    if should_throw_error:
+        with pytest.raises(click.ClickException) as e:
+            cluster_env_controller.archive(cluster_env_name, cluster_env_id)
+
+        assert "Please only provide one of" in e.value.message
+
+        cluster_env_controller.api_client.archive_cluster_environment_api_v2_application_templates_application_template_id_archive_post.assert_not_called()
+    else:
+        with patch.multiple(
+            "anyscale.controllers.cluster_env_controller",
+            get_cluster_env_from_name=mock_get_cluster_env_from_name,
+        ):
+            cluster_env_controller.archive(cluster_env_name, cluster_env_id)
+
+        if cluster_env_id:
+            cluster_env_controller.anyscale_api_client.get_cluster_environment.assert_called_with(
+                cluster_env_id
+            )
+        else:
+            mock_get_cluster_env_from_name.assert_called_with(
+                cluster_env_name, cluster_env_controller.anyscale_api_client
+            )
+
+        cluster_env_controller.api_client.archive_cluster_environment_api_v2_application_templates_application_template_id_archive_post.assert_called_with(
+            "mock_cluster_env_id"
+        )
+
+
 def test_list_builds(mock_auth_api_client,):
     with patch.multiple(
         "anyscale.authenticate.AuthenticationBlock",
