@@ -59,7 +59,8 @@ def example_compute_config() -> Dict[str, Any]:
     "head_node_type": {
         "name": "head-node-type",
         "instance_type": "m5.2xlarge",
-        "resources": null
+        "resources": null,
+        "flags": {}
     },
     "worker_node_types": [
         {
@@ -68,7 +69,8 @@ def example_compute_config() -> Dict[str, Any]:
         "resources": null,
         "min_workers": null,
         "max_workers": 10,
-        "use_spot": false
+        "use_spot": false,
+        "flags": {}
         },
         {
         "name": "worker-node-type-1",
@@ -76,14 +78,17 @@ def example_compute_config() -> Dict[str, Any]:
         "resources": null,
         "min_workers": null,
         "max_workers": 10,
-        "use_spot": false
+        "use_spot": false,
+        "flags": {}
         }
     ],
     "aws_advanced_configurations_json": null,
     "gcp_advanced_configurations_json": null,
     "aws": null,
     "gcp": null,
-    "azure": null
+    "azure": null,
+    "auto_select_worker_config": false,
+    "flags": {},
     }
     """
     return yaml.load(example_compute_config_str, Loader=SafeLoader)
@@ -147,19 +152,28 @@ def test_create_validation(
 
 
 @pytest.mark.parametrize("name", [None, "test_name"])
+@pytest.mark.parametrize("enable_compute_config_versioning", [True, False])
 def test_create(
-    mock_auth_api_client, name: Optional[str], example_compute_config: Dict[str, Any],
+    mock_auth_api_client,
+    name: Optional[str],
+    enable_compute_config_versioning: bool,
+    example_compute_config: Dict[str, Any],
 ) -> None:
     compute_config_controller = ComputeConfigController()
 
     mock_load = Mock(return_value=example_compute_config)
     mock_cluster_compute_io = Mock()
+    compute_config_controller.api_client.check_is_feature_flag_on_api_v2_userinfo_check_is_feature_flag_on_get = Mock(
+        return_value=Mock(result=Mock(is_on=enable_compute_config_versioning))
+    )
     with patch.multiple("yaml", load=mock_load):
         compute_config_controller.create(mock_cluster_compute_io, name)
 
     compute_config_controller.anyscale_api_client.create_cluster_compute.assert_called_once_with(
         CreateClusterCompute(
-            name=ANY, config=compute_config_with_default_fields(example_compute_config)
+            name=ANY,
+            config=compute_config_with_default_fields(example_compute_config),
+            new_version=enable_compute_config_versioning,
         )
     )
 
