@@ -1,4 +1,3 @@
-import asyncio
 import json
 import os
 from pathlib import Path
@@ -8,35 +7,31 @@ from types import SimpleNamespace
 from typing import Any, Dict, List, Optional, Tuple, Type
 from unittest.mock import ANY, call, MagicMock, Mock, mock_open, patch
 
-from aiohttp import ClientResponseError, ClientSession, WSMsgType
-from asynctest import CoroutineMock
+import aiohttp
+from asynctest import CoroutineMock, patch as async_patch
 import click
-from pydantic import ValidationError
 import pytest
 import yaml
 
-from anyscale.api_utils.job_logs_util import _get_job_logs_from_storage_bucket
-from anyscale.api_utils.logs_util import _stream_log_from_ray_websocket
+from anyscale.anyscale_pydantic import ValidationError
 from anyscale.client.openapi_client.models.create_internal_production_job import (
     CreateInternalProductionJob,
 )
 from anyscale.client.openapi_client.models.ha_job_states import HaJobStates
+from anyscale.client.openapi_client.models.job_queue_config import JobQueueConfig
 from anyscale.client.openapi_client.models.production_job_config import (
     ProductionJobConfig,
 )
-from anyscale.controllers import job_controller as job_controller_module
 from anyscale.controllers.job_controller import (
     JobController,
     LogProvider,
     LogsLogger,
+    LokiLogProvider,
 )
 from anyscale.models.job_model import _working_dir_is_remote_uri, JobConfig
 from anyscale.sdk.anyscale_client.models.compute_template import ComputeTemplate
-from anyscale.shared_anyscale_utils.test_util import (
-    get_args_from_call,
-    MockAsyncContextManagerReturnValue,
-)
-from anyscale.util import poll, PROJECT_NAME_ENV_VAR
+from anyscale.shared_anyscale_utils.test_util import MockAsyncContextManagerReturnValue
+from anyscale.util import PROJECT_NAME_ENV_VAR
 from anyscale.utils.runtime_env import is_dir_remote_uri
 
 
@@ -205,6 +200,32 @@ def test_config_validation(add_extra_field, mock_auth_api_client):
         {"entrypoint": "mock_entrypoint", "cloud": "mock_cloud"},
         {"entrypoint": "mock_entrypoint"},
         {"entrypoint": "mock_entrypoint", "project_id": "specified_project_id"},
+        # Use-cases using job queues
+        {
+            "entrypoint": "mock_entrypoint",
+            "build_id": "mock_build_id",
+            "compute_config_id": "cpt_123",
+            "job_queue_config": {
+                "job_queue_spec": {
+                    "job_queue_name": "mock_job_queue",
+                    "idle_timeout_sec": 120,
+                }
+            },
+        },
+        {
+            "entrypoint": "mock_entrypoint",
+            "build_id": "mock_build_id",
+            "compute_config_id": "cpt_123",
+            "job_queue_config": {"target_job_queue_id": "mock_job_queue_id",},
+        },
+        {
+            "entrypoint": "mock_entrypoint",
+            "build_id": "mock_build_id",
+            "compute_config_id": "cpt_123",
+            "job_queue_config": {
+                "target_job_queue_name": "mock_job_queue_user_provided_id",
+            },
+        },
     ],
 )
 @pytest.mark.parametrize("use_default_project", [True, False])
@@ -293,6 +314,13 @@ def test_submit_job(
         final_project_id = "mock_default_project_id"
     else:
         final_project_id = "mock_project_id"
+
+    if "job_queue_config" in config_dict:
+        job_queue_config_dict = config_dict["job_queue_config"]
+        expected_job_queue_config = JobQueueConfig(**job_queue_config_dict)
+    else:
+        expected_job_queue_config = None
+
     job_controller.api_client.create_job_api_v2_decorated_ha_jobs_create_post.assert_called_once_with(
         CreateInternalProductionJob(
             name="mock_name",
@@ -306,6 +334,7 @@ def test_submit_job(
                     "compute_config_id": compute_template_test_data.id,
                 }
             ),
+            job_queue_config=expected_job_queue_config,
         )
     )
     assert mock_logger.info.call_count == 5
@@ -935,7 +964,7 @@ def fake_log_provider(mode: str) -> Type[LogProvider]:
         def close(self):
             pass
 
-        def query(  # noqa: PLR0911
+        async def query(  # noqa: PLR0911
             self, start_timestamp_ns: int, end_timestamp_ns: int
         ) -> List[Tuple[int, str]]:
             # For typing
@@ -994,161 +1023,6 @@ def fake_log_provider(mode: str) -> Type[LogProvider]:
     return FakeLogProvider
 
 
-def test_print_logs_for_active_job_run(mock_auth_api_client) -> None:
-    job_controller = JobController(log=FakeLogger())
-    job_id = "job_123"
-    websocket_url = "http://localhost:5000/stream"
-    job_controller.api_client = Mock(
-        get_job_logs_stream_api_v2_logs_job_logs_stream_job_id_get=Mock(
-            return_value=Mock(result=Mock(stream_url=websocket_url))
-        )
-    )
-    mock_websocket = MockAsyncContextManagerReturnValue()
-
-    # Test logs successfully streamed (followed) from cluster
-    mock_websocket.receive = CoroutineMock(
-        side_effect=[
-            Mock(type=WSMsgType.TEXT, data=" abc "),
-            Mock(type=WSMsgType.ERROR),
-            Mock(type=WSMsgType.CLOSED),
-        ]
-    )
-    with patch.object(
-        ClientSession, ClientSession.ws_connect.__name__, return_value=mock_websocket
-    ):
-        job_controller._print_logs_for_active_job_run(job_id, should_follow=True)
-    job_controller.api_client.get_job_logs_stream_api_v2_logs_job_logs_stream_job_id_get.assert_called_once_with(
-        job_id=job_id
-    )
-    assert mock_websocket.receive.call_count == 3
-    assert job_controller.log.logs == ["abc"]  # type: ignore
-
-    # Test other exception thrown
-    mock_other_exception = ClientResponseError(
-        request_info=Mock(), history=Mock(), status=400
-    )
-    mock_websocket.receive = CoroutineMock(side_effect=mock_other_exception)
-    with patch.object(
-        ClientSession, ClientSession.ws_connect.__name__, return_value=mock_websocket
-    ), pytest.raises(ClientResponseError):
-        job_controller._print_logs_for_active_job_run(job_id, should_follow=True)
-
-    # Test connection issues are raised and logged as error output
-    # Indirectly tests `handle_and_log_lines_in_click_exception()`
-    job_controller.log.error = Mock()  # type: ignore
-    with patch.object(ClientSession, ClientSession.ws_connect.__name__), patch.object(
-        job_controller_module,
-        _stream_log_from_ray_websocket.__name__,
-        new=CoroutineMock(side_effect=asyncio.TimeoutError),
-    ):
-        job_controller._print_logs_for_active_job_run(job_id, should_follow=True)
-    log_content_args = job_controller.log.error.call_args_list[0]
-    log_content: str = get_args_from_call(log_content_args)[0]
-    assert (
-        "only supported if the Anyscale client (SDK / CLI) can reach the cluster"
-        in log_content
-    )
-
-
-def test_print_logs_for_inactive_job_run(mock_auth_api_client) -> None:
-    job_id = "job_123"
-    job_controller = JobController(log=Mock())
-    job_controller.log.spinner = Mock(return_value=MockAsyncContextManagerReturnValue())  # type: ignore
-    log_call_return_vals = ["", "logs\nretrieved"]
-
-    # Test `_get_job_logs_from_storage_bucket`` called twice, and `log.log()` called twice with split logs
-    with patch.object(
-        job_controller_module, poll.__name__, return_value=["bucket", "hit", "twice"]
-    ) as mock_poll, patch.object(
-        job_controller_module,
-        _get_job_logs_from_storage_bucket.__name__,
-        new=CoroutineMock(side_effect=log_call_return_vals),
-    ) as mock_get_job_logs_from_storage_bucket:
-        job_controller._print_logs_for_inactive_job_run(job_id)
-    job_controller.log.spinner.assert_called_once_with("Retrieving logs...")
-    mock_poll.assert_called_once_with(interval_secs=5, max_iter=10)
-    assert mock_get_job_logs_from_storage_bucket.call_count == len(log_call_return_vals)
-    assert job_controller.log.log.call_count == 2  # type: ignore
-    assert get_args_from_call(job_controller.log.log.call_args_list[0])[0] == "logs"  # type: ignore
-    assert (
-        get_args_from_call(job_controller.log.log.call_args_list[1])[0] == "retrieved"  # type: ignore
-    )
-
-    # Test `log.error()` called when logs are empty after polling
-    empty_log_call_return_vals = ["", ""]
-    with patch.object(
-        job_controller_module, poll.__name__, return_value=["tried", "twice"]
-    ) as mock_poll, patch.object(
-        job_controller_module,
-        _get_job_logs_from_storage_bucket.__name__,
-        new=CoroutineMock(side_effect=empty_log_call_return_vals),
-    ) as mock_get_job_logs_from_storage_bucket:
-        job_controller._print_logs_for_inactive_job_run(job_id)
-    assert mock_get_job_logs_from_storage_bucket.call_count == len(
-        empty_log_call_return_vals
-    )
-    job_controller.log.error.assert_called_once_with(  # type: ignore
-        "Logs are not available for this job run."
-    )
-
-
-def test_logs(mock_auth_api_client) -> None:
-    job_controller = JobController()
-    prod_job_id = "prodjob_123"
-    last_job_run_id = "job_123"
-    mock_job = Mock(
-        id=prod_job_id,
-        last_job_run_id=last_job_run_id,
-        state=Mock(current_state=HaJobStates.TERMINATED),
-    )
-    job_controller._wait_for_a_job_run = Mock(return_value=mock_job)  # type: ignore
-    job_controller._print_logs_for_active_job_run = mock_stream_logs = Mock()  # type: ignore
-    job_controller._print_logs_for_inactive_job_run = mock_download_logs = Mock()  # type: ignore
-    mock_ff_response = Mock()
-    mock_ff_response.result = Mock()
-    mock_ff_response.result.is_on = False
-    job_controller.api_client.check_is_feature_flag_on_api_v2_userinfo_check_is_feature_flag_on_get = Mock(
-        return_value=mock_ff_response
-    )
-
-    def reset_mocks() -> None:
-        for mock in [mock_stream_logs, mock_download_logs]:
-            mock.reset_mock()
-
-    def do_assertions(
-        called: Optional[List[Mock]] = None, not_called: Optional[List[Mock]] = None
-    ) -> None:
-        if not called:
-            called = []
-        if not not_called:
-            not_called = []
-        for mock in called:
-            mock.assert_called_once()
-        for mock in not_called:
-            mock.assert_not_called()
-
-    # Test default if job is not in RUNNING state
-    job_controller.logs(job_id=prod_job_id)
-    do_assertions(not_called=[mock_stream_logs])
-    mock_download_logs.assert_called_once_with(last_job_run_id)
-
-    # Test stream if job is RUNNING
-    reset_mocks()
-    mock_job.state.current_state = HaJobStates.RUNNING
-    job_controller.logs(job_id=prod_job_id)
-    do_assertions(called=[mock_stream_logs], not_called=[mock_download_logs])
-
-    # Test `all_attempts = True`
-    reset_mocks()
-    job_controller._get_all_job_runs = Mock(  # type: ignore
-        return_value=[Mock(id="job_1"), Mock(id=last_job_run_id),]
-    )
-    job_controller.logs(job_id=prod_job_id, should_follow=True, all_attempts=True)
-    job_controller._get_all_job_runs.assert_called_once_with(prod_job_id)
-    mock_download_logs.assert_called_once_with("job_1")
-    mock_stream_logs.assert_called_once_with(last_job_run_id, True)
-
-
 def test_logs_streaming(mock_auth_api_client) -> None:
     job_controller = JobController()
     prod_job_id = "prodjob_123"
@@ -1430,3 +1304,71 @@ def test_override_runtime_env_config_and_infer_project_id(mock_auth_api_client):
                 ),
             )
         )
+
+
+async def test_loki_log_provider():
+    tenant_id = "fake_tenant_id"
+    token = "fake_token"
+    mocked_response = MockAsyncContextManagerReturnValue()
+    mocked_response.json = CoroutineMock(
+        return_value={
+            "status": "success",
+            "data": {
+                "resultType": "streams",
+                "result": [
+                    {
+                        "metric": {"level": "info"},
+                        "values": [[1, "log1"], [3, "log3"], [4, "log4"],],
+                    },
+                    {
+                        "metric": {"level": "warn"},
+                        "values": [[2, "log2"], [6, "log6"], [5, "log5"],],
+                    },
+                ],
+            },
+        }
+    )
+    mocked_session = MagicMock()
+    mocked_session.get.return_value = mocked_response
+    mocked_session.close = CoroutineMock(return_value=None)
+    with async_patch.object(
+        aiohttp, aiohttp.ClientSession.__name__, return_value=mocked_session,
+    ):
+        async with LokiLogProvider(
+            dns_name="fake_dns_name",
+            token=token,
+            query="fake_query",
+            limit=100,
+            tenant_id=tenant_id,
+            direction="backward",
+        ) as loki_log_provider:
+            logs = await loki_log_provider.query(
+                start_timestamp_ns=123, end_timestamp_ns=456
+            )
+            expected_logs = [
+                (1, "log1"),
+                (3, "log3"),
+                (4, "log4"),
+                (2, "log2"),
+                (6, "log6"),
+                (5, "log5"),
+            ]
+            assert logs == expected_logs
+
+        # Ensure that the headers are set correctly
+        expected_headers = {
+            "X-Scope-OrgID": tenant_id,
+            "Authorization": f"Bearer {token}",
+        }
+        mocked_session.get.assert_called_once_with(
+            "https://fake_dns_name/loki/api/v1/query_range",
+            headers=expected_headers,
+            params={
+                "query": "fake_query",
+                "limit": 100,
+                "start": 123,
+                "end": 456,
+                "direction": "backward",
+            },
+            timeout=60,
+        )
