@@ -1,7 +1,5 @@
 from unittest.mock import Mock
 
-import pytest
-
 from anyscale.client.openapi_client.models import (
     ListMachinesResponse,
     MachineAllocationState,
@@ -11,8 +9,7 @@ from anyscale.client.openapi_client.models import (
 from frontend.cli.anyscale.controllers.machine_controller import MachineController
 
 
-@pytest.mark.parametrize("cloud_id", ["cld_123"])
-def test_list_machines(mock_auth_api_client, cloud_id: str) -> None:
+def test_list_machines(mock_auth_api_client: str) -> None:
     response = ListMachinesResponse(
         machines=[
             MachineInfo(
@@ -32,5 +29,7 @@ def test_list_machines(mock_auth_api_client, cloud_id: str) -> None:
     machine_controller.api_client.list_machines_api_v2_machines_get = Mock(
         return_value=api_response
     )
-    output = machine_controller.list_machines(cloud_id=cloud_id, cloud=None)
+    output = machine_controller.list_machines(
+        cloud_id=None, cloud=None, machine_pool_name="pool1"
+    )
     assert output == response
