@@ -162,19 +162,23 @@ def test_get_project_directory_name(project_test_data: Project) -> None:
 
 def test_get_working_dir(project_test_data: Project) -> None:
     mock_api_client = Mock()
-    mock_api_client.get_project_api_v2_projects_project_id_get.return_value.result.directory_name = (
-        project_test_data.directory_name
+    mock_project = Mock(directory_name=project_test_data.directory_name)
+    mock_project.name = "mock_proj_name"
+    mock_api_client.get_project_api_v2_projects_project_id_get.return_value.result = (
+        mock_project
+    )
+    mock_api_client.check_is_feature_flag_on_api_v2_userinfo_check_is_feature_flag_on_get.return_value.result.is_on = (
+        True
     )
 
-    working_dir = get_working_dir({}, project_test_data.id, mock_api_client)
-    assert working_dir == f"/home/ray/{project_test_data.directory_name}"
+    working_dir = get_working_dir(project_test_data.id, mock_api_client)
+    assert working_dir == f"/home/ray/{mock_project.directory_name}"
 
-    working_dir = get_working_dir(
-        {"metadata": {"anyscale": {"working_dir": "test_working_dir"}}},
-        project_test_data.id,
-        mock_api_client,
+    mock_api_client.check_is_feature_flag_on_api_v2_userinfo_check_is_feature_flag_on_get.return_value.result.is_on = (
+        False
     )
-    assert working_dir == "test_working_dir"
+    working_dir = get_working_dir(project_test_data.id, mock_api_client,)
+    assert working_dir == f"/home/ray/{mock_project.name}"
 
 
 def test_credentials_check_sanity():
