@@ -32,6 +32,7 @@ from anyscale.client.openapi_client.models.subnet_id_with_availability_zone_aws
     SubnetIdWithAvailabilityZoneAWS,
 )
 from anyscale.cloud_resource import (
+    _verify_aws_efs_policy,
     is_internal_communication_allowed,
     verify_aws_cloudformation_stack,
     verify_aws_efs,
@@ -44,7 +45,7 @@ from anyscale.cloud_resource import (
 )
 from anyscale.conf import ANYSCALE_IAM_ROLE_NAME
 from anyscale.shared_anyscale_utils.aws import AwsRoleArn
-from anyscale.shared_anyscale_utils.conf import ANYSCALE_HOST
+from anyscale.shared_anyscale_utils.conf import ANYSCALE_CORS_ORIGIN
 from anyscale.utils.network_verification import AWS_SUBNET_CAPACITY
 
 
@@ -1099,7 +1100,7 @@ def test_verify_aws_security_groups_multiple_security_groups(
                 {
                     "AllowedHeaders": ["*"],
                     "AllowedMethods": ["GET"],
-                    "AllowedOrigins": [ANYSCALE_HOST],
+                    "AllowedOrigins": [ANYSCALE_CORS_ORIGIN],
                     "ExposeHeaders": [],
                 }
             ],
@@ -1237,22 +1238,88 @@ def test_verify_aws_efs(
         cloud_resource_mock.aws_security_groups = [security_group_id]
         cloud_resource_mock.aws_efs_id = efs_id
 
-    result = verify_aws_efs(
-        cloud_resource=cloud_resource_mock,
-        boto3_session=boto3.Session(region_name=region),
-        logger=CloudSetupLogger(),
-    )
-    _, stderr = capsys.readouterr()
-    if warning:
-        assert re.search(r"does not contain a mount target with the subnet", stderr)
-    if enable_backup:
-        assert not re.search(r"backup policy is not enabled", stderr)
-        assert not re.search(r"backup policy not found", stderr)
-    else:
-        assert re.search(r"backup policy is not enabled", stderr) or re.search(
-            r"backup policy not found", stderr
+    with patch.multiple(
+        "anyscale.cloud_resource", _verify_aws_efs_policy=Mock(return_value=True),
+    ):
+        result = verify_aws_efs(
+            cloud_resource=cloud_resource_mock,
+            boto3_session=boto3.Session(region_name=region),
+            logger=CloudSetupLogger(),
         )
-    assert result == expected_result
+        _, stderr = capsys.readouterr()
+        if warning:
+            assert re.search(r"does not contain a mount target with the subnet", stderr)
+        if enable_backup:
+            assert not re.search(r"backup policy is not enabled", stderr)
+            assert not re.search(r"backup policy not found", stderr)
+        else:
+            assert re.search(r"backup policy is not enabled", stderr) or re.search(
+                r"backup policy not found", stderr
+            )
+        assert result == expected_result
+
+
+@pytest.mark.parametrize("strict", [True, False])
+@pytest.mark.parametrize(
+    "policy",
+    [
+        None,
+        [
+            "elasticfilesystem:ClientRootAccess",
+            "elasticfilesystem:ClientWrite",
+            "elasticfilesystem:ClientMount",
+        ],
+        ["elasticfilesystem:ClientWrite", "elasticfilesystem:ClientMount"],
+    ],
+)
+def test_verify_aws_efs_policy(strict: bool, policy: Optional[List[str]], capsys):
+    mock_boto3 = Mock()
+    efs_id = "mock-efs"
+
+    mock_efs_client = Mock()
+    if policy:
+        mock_policy_dict = {
+            "Version": "2012-10-17",
+            "Id": "efs-policy",
+            "Statement": [
+                {
+                    "Sid": "efs-statement",
+                    "Effect": "Allow",
+                    "Principal": {"AWS": "*"},
+                    "Action": policy,
+                    "Condition": {
+                        "Bool": {"elasticfilesystem:AccessedViaMountTarget": "true"}
+                    },
+                }
+            ],
+        }
+        mock_efs_client.describe_file_system_policy = Mock(
+            return_value={"Policy": json.dumps(mock_policy_dict)}
+        )
+    else:
+        mock_efs_client.describe_file_system_policy = Mock(
+            side_effect=ClientError(
+                error_response={"Error": {"Code": "PolicyNotFound"}},
+                operation_name="describe_file_system_policy",
+            )
+        )
+
+    mock_boto3.client = Mock(return_value=mock_efs_client)
+    verification_result = _verify_aws_efs_policy(
+        mock_boto3, efs_id, CloudSetupLogger(), strict
+    )
+
+    _, stderr = capsys.readouterr()
+    if not policy:
+        assert verification_result is True
+        assert "does not have sufficient permissions" not in stderr
+    else:
+        if len(policy) == 3:
+            assert verification_result is True
+            assert "does not have sufficient permissions" not in stderr
+        else:
+            assert verification_result == (not strict)
+            assert "does not have sufficient permissions" in stderr
 
 
 @pytest.mark.parametrize(
@@ -1297,11 +1364,14 @@ def test_verify_aws_efs_valid_ip(
     else:
         cloud_resource_mock.aws_efs_mount_target_ip = "1.2.3.4"
 
-    result = verify_aws_efs(
-        cloud_resource=cloud_resource_mock,
-        boto3_session=boto3.Session(region_name=region),
-        logger=CloudSetupLogger(),
-    )
+    with patch.multiple(
+        "anyscale.cloud_resource", _verify_aws_efs_policy=Mock(return_value=True),
+    ):
+        result = verify_aws_efs(
+            cloud_resource=cloud_resource_mock,
+            boto3_session=boto3.Session(region_name=region),
+            logger=CloudSetupLogger(),
+        )
 
     stdout, stderr = capsys.readouterr()
     sys.stdout.write(stdout)
