@@ -15,6 +15,7 @@ from anyscale.utils.gcp_utils import (
     get_application_default_credentials,
     get_filestore_location_and_instance_id,
     get_gcp_filestore_config,
+    get_gcp_memorystore_config,
     get_google_cloud_client_factory,
 )
 
@@ -113,6 +114,34 @@ def test_get_gcp_filestore_config(
             assert "This cannot be edited on an existing Filestore instance." in err
 
 
+@pytest.mark.parametrize(
+    ("enable_head_node_fault_tolerance", "response"),
+    [
+        pytest.param(True, ("200 OK", "regional_memorystore.json"),),
+        pytest.param(False, (None, None),),
+    ],
+)
+def test_get_gcp_memorystore_config(
+    setup_mock_server, enable_head_node_fault_tolerance, response
+):
+    factory, tracker = setup_mock_server
+    tracker.reset({".*": [response]})
+    mock_redis_instance_name = (
+        "projects/mock_project/locations/us-west1/instances/mock_redis_instance_name"
+        if enable_head_node_fault_tolerance
+        else None
+    )
+    memorystore_instance_config = get_gcp_memorystore_config(
+        factory, mock_redis_instance_name
+    )
+
+    if enable_head_node_fault_tolerance:
+        assert memorystore_instance_config is not None
+        assert memorystore_instance_config.name == mock_redis_instance_name
+    else:
+        assert memorystore_instance_config is None
+
+
 def test_get_application_default_credentials_success(capsys):
     mock_run = Mock()
     with patch(
