@@ -1,4 +1,5 @@
 import os
+from typing import Optional
 from unittest.mock import ANY, Mock, mock_open, patch
 
 import click
@@ -307,7 +308,8 @@ def test_get_default_project(cloud_isolation_phase_1_feature_flag: bool):
     )
 
 
-def test_infer_project_id():
+@pytest.mark.parametrize("compute_config_version", [None, 2])
+def test_infer_project_id(compute_config_version: Optional[int]):
     # Test passing compute config
     anyscale_api_client = Mock()
     api_client = Mock()
@@ -341,6 +343,43 @@ def test_infer_project_id():
         cloud_name=None,
     )
 
+    # Test passing compute config name
+    anyscale_api_client = Mock()
+    api_client = Mock()
+    mock_cluster_compute = Mock(id="mock_cluster_compute_id")
+    mock_get_default_project = Mock(return_value=Mock(id="mock_project_id1"))
+    mock_get_selected_cloud_id_or_default = Mock(return_value="mock_cloud_id1")
+    mock_get_cluster_compute_from_name = Mock(return_value=mock_cluster_compute)
+    with patch.multiple(
+        "anyscale.project",
+        get_default_project=mock_get_default_project,
+        find_project_root=Mock(return_value=None),
+        get_selected_cloud_id_or_default=mock_get_selected_cloud_id_or_default,
+        get_cluster_compute_from_name=mock_get_cluster_compute_from_name,
+    ):
+        assert (
+            infer_project_id(
+                anyscale_api_client,
+                api_client,
+                Mock(),
+                project_id=None,
+                cluster_compute="mock_cluster_compute_name",
+                cluster_compute_version=compute_config_version,
+            )
+            == "mock_project_id1"
+        )
+    mock_get_default_project.assert_called_once_with(
+        api_client, anyscale_api_client, parent_cloud_id="mock_cloud_id1"
+    )
+    mock_get_selected_cloud_id_or_default.assert_called_once_with(
+        api_client,
+        anyscale_api_client,
+        cluster_compute_id="mock_cluster_compute_id",
+        cluster_compute_config=None,
+        cloud_id=None,
+        cloud_name=None,
+    )
+
     # Test passing in cloud
     mock_get_default_project = Mock(return_value=Mock(id="mock_project_id2"))
     mock_get_selected_cloud_id_or_default = Mock(return_value="mock_cloud_id2")
