@@ -1,6 +1,7 @@
 # type: ignore
 
 import os
+from tempfile import NamedTemporaryFile
 from typing import Any, Dict, Optional
 from unittest.mock import Mock, mock_open, patch
 
@@ -15,7 +16,7 @@ from anyscale.client.openapi_client.models.decoratedsession_response import (
 from anyscale.utils.runtime_env import (
     _get_cloud_gs_bucket_from_cloud,
     _get_cloud_s3_bucket_from_cloud,
-    autopopulate_working_dir_for_workspace,
+    autopopulate_runtime_env_for_workspace,
     infer_upload_path_and_rewrite_working_dir,
     override_runtime_env_config,
 )
@@ -225,7 +226,7 @@ class TestRuntimeEnvOverride:
                 log=mock_log,
             )
 
-            runtime_env = autopopulate_working_dir_for_workspace(
+            runtime_env = autopopulate_runtime_env_for_workspace(
                 runtime_env=runtime_env, log=Mock()
             )
 
@@ -290,47 +291,106 @@ class TestRuntimeEnvOverride:
 
 
 @pytest.mark.parametrize(
-    ("runtime_env", "is_workspace", "expected_runtime_env"),
+    (
+        "runtime_env",
+        "is_workspace",
+        "requirements_file_contents",
+        "expected_runtime_env",
+    ),
     [
-        [None, False, None],
-        [{}, False, {}],
+        # If not in a workspace, the fields should not be modified.
+        [None, False, None, None],
+        [{}, False, None, {}],
         [
-            {"working_dir": "job-services-cuj-examples"},
+            {"working_dir": "s3://something.zip", "pip": ["test"]},
             False,
-            {"working_dir": "job-services-cuj-examples"},
+            None,
+            {"working_dir": "s3://something.zip", "pip": ["test"]},
         ],
-        [None, True, {"working_dir": "."}],
-        [{}, True, {"working_dir": "."}],
+        # If no working_dir is passed, it should default to ".".
+        [None, True, None, {"working_dir": "."}],
+        [{}, True, None, {"working_dir": "."}],
+        # If a working_dir is passed, it should not be altered.
         [
-            {"working_dir": "job-services-cuj-examples"},
+            {"working_dir": "local-path-from-user"},
             True,
-            {"working_dir": "job-services-cuj-examples"},
+            None,
+            {"working_dir": "local-path-from-user"},
+        ],
+        [
+            {"working_dir": "s3://remote-path-from-user.zip"},
+            True,
+            None,
+            {"working_dir": "s3://remote-path-from-user.zip"},
+        ],
+        # If no pip or conda field is specified, the requirements file should be picked up.
+        # Also check that the file is read and stripped of comments & whitespace.
+        [
+            {},
+            True,
+            "test-pkg1\n test-pkg2==0.1.2\n \n#comment\n ",
+            {"working_dir": ".", "pip": ["test-pkg1", "test-pkg2==0.1.2"]},
+        ],
+        [
+            {},
+            True,
+            "test-pkg1\n test-pkg2==0.1.2#comment\n ",
+            {"working_dir": ".", "pip": ["test-pkg1", "test-pkg2==0.1.2"]},
+        ],
+        [
+            {},
+            True,
+            "test-pkg1\n test-pkg2==0.1.2 #comment\n ",
+            {"working_dir": ".", "pip": ["test-pkg1", "test-pkg2==0.1.2"]},
+        ],
+        # If the requirements file is empty (after stripping), the field shouldn't be set.
+        [{}, True, "", {"working_dir": "."},],
+        [{}, True, "\n#comment\n ", {"working_dir": "."},],
+        # If the pip or conda field is specified, it should not be altered.
+        [
+            {"pip": ["some-other-pkg"]},
+            True,
+            "test-pkg1\ntest-pkg2\n",
+            {"working_dir": ".", "pip": ["some-other-pkg"]},
+        ],
+        [
+            {"conda": ["some-other-pkg"]},
+            True,
+            "test-pkg1\ntest-pkg2\n",
+            {"working_dir": ".", "conda": ["some-other-pkg"]},
         ],
     ],
 )
-def test_autopopulate_working_dir_for_workspace(
+def test_autopopulate_runtime_env_for_workspace(
     runtime_env: Optional[Dict[str, Any]],
     is_workspace: bool,
+    requirements_file_contents: Optional[str],
     expected_runtime_env: Optional[Dict[str, Any]],
 ):
-    """Test autopopulate working_dir if deployed from workspace
-    If non-workspace environment return existing runtime_env
-    """
-
+    """Test that working_dir and pip fields are populated properly for workspaces."""
     mock_logger = Mock()
-
-    if is_workspace:
-        mock_os_environ = {
+    mock_os_environ = (
+        {
             "ANYSCALE_SESSION_ID": "fake_session_id",
             "ANYSCALE_EXPERIMENTAL_WORKSPACE_ID": "fake_workspace_id",
+            "ANYSCALE_WORKSPACE_DYNAMIC_DEPENDENCY_TRACKING": "1",
         }
-        with patch.dict(os.environ, mock_os_environ):
-            modified_runtime_env = autopopulate_working_dir_for_workspace(
-                runtime_env, mock_logger,
-            )
+        if is_workspace
+        else {}
+    )
+
+    requirements_file_path = None
+    if requirements_file_contents is not None:
+        f = NamedTemporaryFile("w")
+        f.write(requirements_file_contents)
+        f.flush()
+        requirements_file_path = f.name
     else:
-        modified_runtime_env = autopopulate_working_dir_for_workspace(
-            runtime_env, mock_logger
+        requirements_file_path = "/some/nonexistent/path"
+
+    with patch.dict(os.environ, mock_os_environ):
+        modified_runtime_env = autopopulate_runtime_env_for_workspace(
+            runtime_env, mock_logger, requirements_file_path=requirements_file_path,
         )
 
     assert modified_runtime_env == expected_runtime_env
